@@ -23,7 +23,6 @@
 #include <windows.h>
 #if defined(_WIN32) && !defined(_XBOX)
 #include <process.h>
-#include <errno.h>
 #endif
 
 #include <boolean.h>
@@ -46,19 +45,12 @@
 #include "../frontend_driver.h"
 #include "../../configuration.h"
 #include "../../defaults.h"
-#include "../../verbosity.h"
-#include "../../ui/drivers/ui_win32.h"
 #include "../../paths.h"
 #include "../../msg_hash.h"
-#include "platform_win32.h"
-
 #include "../../verbosity.h"
+#include "../../ui/drivers/ui_win32.h"
 
-/*
-#ifdef HAVE_NVDA
-#include "../../nvda_controller.h"
-#endif
-*/
+#include "platform_win32.h"
 
 #ifdef HAVE_SAPI
 #define COBJMACROS
@@ -66,38 +58,42 @@
 #include <ole2.h>
 #endif
 
-#ifdef HAVE_SAPI
-static ISpVoice* pVoice        = NULL;
-#endif
-#ifdef HAVE_NVDA
-static bool USE_POWERSHELL     = false;
-static bool USE_NVDA           = true;
-#else
-static bool USE_POWERSHELL     = true;
-static bool USE_NVDA           = false;
-#endif
-static bool USE_NVDA_BRAILLE   = false;
-
 #ifndef SM_SERVERR2
 #define SM_SERVERR2 89
 #endif
 
+enum platform_win32_flags
+{
+   PLAT_WIN32_FLAG_USE_POWERSHELL           = (1 << 0),
+   PLAT_WIN32_FLAG_USE_NVDA                 = (1 << 1),
+   PLAT_WIN32_FLAG_USE_NVDA_BRAILLE         = (1 << 2),
+   PLAT_WIN32_FLAG_DWM_COMPOSITION_DISABLED = (1 << 3),
+   PLAT_WIN32_FLAG_CONSOLE_NEEDS_FREE       = (1 << 4),
+   PLAT_WIN32_FLAG_PROCESS_INSTANCE_SET     = (1 << 5)
+};
+
+#ifdef HAVE_SAPI
+static ISpVoice *voice_ptr        = NULL;
+#endif
+#ifdef HAVE_NVDA
+static uint8_t g_plat_win32_flags = PLAT_WIN32_FLAG_USE_NVDA;
+#else
+static uint8_t g_plat_win32_flags = PLAT_WIN32_FLAG_USE_POWERSHELL;
+#endif
+
 /* static public global variable */
 VOID (WINAPI *DragAcceptFiles_func)(HWND, BOOL);
 
 /* TODO/FIXME - static global variables */
-static bool dwm_composition_disabled = false;
-static bool console_needs_free       = false;
 static char win32_cpu_model_name[64] = {0};
-static bool pi_set                   = false;
-#ifdef HAVE_DYNAMIC
+#ifdef HAVE_DYLIB
 /* We only load this library once, so we let it be
  * unloaded at application shutdown, since unloading
  * it early seems to cause issues on some systems.
  */
-static dylib_t dwmlib;
-static dylib_t shell32lib;
-static dylib_t nvdalib;
+static dylib_t dwm_lib;
+static dylib_t shell32_lib;
+static dylib_t nvda_lib;
 #endif
 
 /* Dynamic loading for Non-Visual Desktop Access support */
@@ -120,32 +116,32 @@ const struct win32_lang_pair win32_lang_pairs[] =
    /* array order MUST be kept, always largest ID first */
    {0x7c04, RETRO_LANGUAGE_CHINESE_TRADITIONAL}, /* neutral */
    {0x1404, RETRO_LANGUAGE_CHINESE_TRADITIONAL}, /* MO */
-   {0x1004, RETRO_LANGUAGE_CHINESE_SIMPLIFIED}, /* SG */
-   {0xC04, RETRO_LANGUAGE_CHINESE_TRADITIONAL}, /* HK/PRC */
-   {0x816, RETRO_LANGUAGE_PORTUGUESE_PORTUGAL},
-   {0x416, RETRO_LANGUAGE_PORTUGUESE_BRAZIL},
-   {0x2a, RETRO_LANGUAGE_VIETNAMESE},
-   {0x19, RETRO_LANGUAGE_RUSSIAN},
-   {0x16, RETRO_LANGUAGE_PORTUGUESE_PORTUGAL},
-   {0x15, RETRO_LANGUAGE_POLISH},
-   {0x13, RETRO_LANGUAGE_DUTCH},
-   {0x12, RETRO_LANGUAGE_KOREAN},
-   {0x11, RETRO_LANGUAGE_JAPANESE},
-   {0x10, RETRO_LANGUAGE_ITALIAN},
-   {0xc, RETRO_LANGUAGE_FRENCH},
-   {0xa, RETRO_LANGUAGE_SPANISH},
-   {0x9, RETRO_LANGUAGE_ENGLISH},
-   {0x8, RETRO_LANGUAGE_GREEK},
-   {0x7, RETRO_LANGUAGE_GERMAN},
-   {0x4, RETRO_LANGUAGE_CHINESE_SIMPLIFIED}, /* neutral */
-   {0x1, RETRO_LANGUAGE_ARABIC},
+   {0x1004, RETRO_LANGUAGE_CHINESE_SIMPLIFIED},  /* SG */
+   {0xC04,  RETRO_LANGUAGE_CHINESE_TRADITIONAL}, /* HK/PRC */
+   {0x816,  RETRO_LANGUAGE_PORTUGUESE_PORTUGAL},
+   {0x416,  RETRO_LANGUAGE_PORTUGUESE_BRAZIL},
+   {0x2a,   RETRO_LANGUAGE_VIETNAMESE},
+   {0x19,   RETRO_LANGUAGE_RUSSIAN},
+   {0x16,   RETRO_LANGUAGE_PORTUGUESE_PORTUGAL},
+   {0x15,   RETRO_LANGUAGE_POLISH},
+   {0x13,   RETRO_LANGUAGE_DUTCH},
+   {0x12,   RETRO_LANGUAGE_KOREAN},
+   {0x11,   RETRO_LANGUAGE_JAPANESE},
+   {0x10,   RETRO_LANGUAGE_ITALIAN},
+   {0xc,    RETRO_LANGUAGE_FRENCH},
+   {0xa,    RETRO_LANGUAGE_SPANISH},
+   {0x9,    RETRO_LANGUAGE_ENGLISH},
+   {0x8,    RETRO_LANGUAGE_GREEK},
+   {0x7,    RETRO_LANGUAGE_GERMAN},
+   {0x4,    RETRO_LANGUAGE_CHINESE_SIMPLIFIED},  /* neutral */
+   {0x1,    RETRO_LANGUAGE_ARABIC},
    /* MS does not support Esperanto */
    /*{0x0, RETRO_LANGUAGE_ESPERANTO},*/
 };
 
 unsigned short win32_get_langid_from_retro_lang(enum retro_language lang)
 {
-   unsigned i;
+   int i;
 
    for (i = 0; i < sizeof(win32_lang_pairs) / sizeof(win32_lang_pairs[0]); i++)
    {
@@ -158,7 +154,7 @@ unsigned short win32_get_langid_from_retro_lang(enum retro_language lang)
 
 enum retro_language win32_get_retro_lang_from_langid(unsigned short langid)
 {
-   unsigned i;
+   int i;
 
    for (i = 0; i < sizeof(win32_lang_pairs) / sizeof(win32_lang_pairs[0]); i++)
    {
@@ -191,13 +187,13 @@ enum retro_language win32_get_retro_lang_from_langid(unsigned short langid)
 
 static void gfx_dwm_shutdown(void)
 {
-#ifdef HAVE_DYNAMIC
-   if (dwmlib)
-      dylib_close(dwmlib);
-   if (shell32lib)
-      dylib_close(shell32lib);
-   dwmlib     = NULL;
-   shell32lib = NULL;
+#ifdef HAVE_DYLIB
+   if (dwm_lib)
+      dylib_close(dwm_lib);
+   if (shell32_lib)
+      dylib_close(shell32_lib);
+   dwm_lib     = NULL;
+   shell32_lib = NULL;
 #endif
 }
 
@@ -211,30 +207,25 @@ static bool gfx_init_dwm(void)
 
    atexit(gfx_dwm_shutdown);
 
-#ifdef HAVE_DYNAMIC
-   shell32lib = dylib_load("shell32.dll");
-   if (!shell32lib)
+#ifdef HAVE_DYLIB
+   if (!(shell32_lib = dylib_load("shell32.dll")))
    {
       RARCH_WARN("Did not find shell32.dll.\n");
    }
 
-   dwmlib = dylib_load("dwmapi.dll");
-   if (!dwmlib)
+   if (!(dwm_lib = dylib_load("dwmapi.dll")))
    {
       RARCH_WARN("Did not find dwmapi.dll.\n");
       return false;
    }
 
    DragAcceptFiles_func =
-      (VOID (WINAPI*)(HWND, BOOL))dylib_proc(shell32lib, "DragAcceptFiles");
+      (VOID (WINAPI*)(HWND, BOOL))dylib_proc(shell32_lib, "DragAcceptFiles");
 
    mmcss =
-      (HRESULT(WINAPI*)(BOOL))dylib_proc(dwmlib, "DwmEnableMMCSS");
+      (HRESULT(WINAPI*)(BOOL))dylib_proc(dwm_lib, "DwmEnableMMCSS");
 #else
    DragAcceptFiles_func = DragAcceptFiles;
-#if 0
-   mmcss                = DwmEnableMMCSS;
-#endif
 #endif
 
    if (mmcss)
@@ -254,12 +245,13 @@ static void gfx_set_dwm(void)
    if (!gfx_init_dwm())
       return;
 
-   if (disable_composition == dwm_composition_disabled)
+   if (disable_composition == (g_plat_win32_flags &
+            PLAT_WIN32_FLAG_DWM_COMPOSITION_DISABLED))
       return;
 
-#ifdef HAVE_DYNAMIC
+#ifdef HAVE_DYLIB
    composition_enable =
-      (HRESULT (WINAPI*)(UINT))dylib_proc(dwmlib, "DwmEnableComposition");
+      (HRESULT (WINAPI*)(UINT))dylib_proc(dwm_lib, "DwmEnableComposition");
 #endif
 
    if (!composition_enable)
@@ -271,7 +263,8 @@ static void gfx_set_dwm(void)
    ret = composition_enable(!disable_composition);
    if (FAILED(ret))
       RARCH_ERR("Failed to set composition state ...\n");
-   dwm_composition_disabled = disable_composition;
+   if (disable_composition)
+      g_plat_win32_flags |= PLAT_WIN32_FLAG_DWM_COMPOSITION_DISABLED;
 }
 
 static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
@@ -329,37 +322,39 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
    switch (vi.dwMajorVersion)
    {
       case 10:
-         if (server)
-            strcpy_literal(s, "Windows Server 2016");
+         if (atoi(buildStr) >= 21996)
+            strlcpy(s, "Windows 11", len);
+         else if (server)
+            strlcpy(s, "Windows Server 2016", len);
          else
-            strcpy_literal(s, "Windows 10");
+            strlcpy(s, "Windows 10", len);
          break;
       case 6:
          switch (vi.dwMinorVersion)
          {
             case 3:
                if (server)
-                  strcpy_literal(s, "Windows Server 2012 R2");
+                  strlcpy(s, "Windows Server 2012 R2", len);
                else
-                  strcpy_literal(s, "Windows 8.1");
+                  strlcpy(s, "Windows 8.1", len);
                break;
             case 2:
                if (server)
-                  strcpy_literal(s, "Windows Server 2012");
+                  strlcpy(s, "Windows Server 2012", len);
                else
-                  strcpy_literal(s, "Windows 8");
+                  strlcpy(s, "Windows 8", len);
                break;
             case 1:
                if (server)
-                  strcpy_literal(s, "Windows Server 2008 R2");
+                  strlcpy(s, "Windows Server 2008 R2", len);
                else
-                  strcpy_literal(s, "Windows 7");
+                  strlcpy(s, "Windows 7", len);
                break;
             case 0:
                if (server)
-                  strcpy_literal(s, "Windows Server 2008");
+                  strlcpy(s, "Windows Server 2008", len);
                else
-                  strcpy_literal(s, "Windows Vista");
+                  strlcpy(s, "Windows Vista", len);
                break;
             default:
                break;
@@ -371,7 +366,7 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
             case 2:
                if (server)
                {
-                  strcpy_literal(s, "Windows Server 2003");
+                  strlcpy(s, "Windows Server 2003", len);
                   if (GetSystemMetrics(SM_SERVERR2))
                      strlcat(s, " R2", len);
                }
@@ -379,14 +374,14 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
                {
                   /* Yes, XP Pro x64 is a higher version number than XP x86 */
                   if (string_is_equal(arch, "x64"))
-                     strcpy_literal(s, "Windows XP");
+                     strlcpy(s, "Windows XP", len);
                }
                break;
             case 1:
-               strcpy_literal(s, "Windows XP");
+               strlcpy(s, "Windows XP", len);
                break;
             case 0:
-               strcpy_literal(s, "Windows 2000");
+               strlcpy(s, "Windows 2000", len);
                break;
          }
          break;
@@ -395,17 +390,17 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
          {
             case 0:
                if (vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
-                  strcpy_literal(s, "Windows 95");
+                  strlcpy(s, "Windows 95", len);
                else if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT)
-                  strcpy_literal(s, "Windows NT 4.0");
+                  strlcpy(s, "Windows NT 4.0", len);
                else
-                  strcpy_literal(s, "Unknown");
+                  strlcpy(s, "Unknown", len);
                break;
             case 90:
-               strcpy_literal(s, "Windows ME");
+               strlcpy(s, "Windows ME", len);
                break;
             case 10:
-               strcpy_literal(s, "Windows 98");
+               strlcpy(s, "Windows 98", len);
                break;
          }
          break;
@@ -435,7 +430,7 @@ static void frontend_win32_init(void *data)
 {
    typedef BOOL (WINAPI *isProcessDPIAwareProc)();
    typedef BOOL (WINAPI *setProcessDPIAwareProc)();
-#ifdef HAVE_DYNAMIC
+#ifdef HAVE_DYLIB
    HMODULE handle                         =
       GetModuleHandle("User32.dll");
    isProcessDPIAwareProc  isDPIAwareProc  =
@@ -457,28 +452,28 @@ static void frontend_win32_init(void *data)
 #ifdef HAVE_NVDA
 static void init_nvda(void)
 {
-#ifdef HAVE_DYNAMIC
-   if (USE_NVDA && !nvdalib)
+#ifdef HAVE_DYLIB
+   if (     (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA) 
+         && !nvda_lib)
    {
-      nvdalib = dylib_load("nvdaControllerClient64.dll");
-      if (!nvdalib)
+      if ((nvda_lib = dylib_load("nvdaControllerClient64.dll")))
       {
-         USE_NVDA = false;
-         USE_POWERSHELL = true;
-      }
-      else
-      {
-         nvdaController_testIfRunning_func = ( unsigned long (__stdcall*)(void))dylib_proc(nvdalib, "nvdaController_testIfRunning");
-         nvdaController_cancelSpeech_func = (unsigned long(__stdcall *)(void))dylib_proc(nvdalib, "nvdaController_cancelSpeech");
-         nvdaController_brailleMessage_func = (unsigned long(__stdcall *)(wchar_t*))dylib_proc(nvdalib, "nvdaController_brailleMessage");
-         nvdaController_speakText_func = (unsigned long(__stdcall *)(wchar_t*))dylib_proc(nvdalib, "nvdaController_speakText");
-      
+         nvdaController_testIfRunning_func  = (unsigned long (__stdcall*)(void))dylib_proc(nvda_lib, "nvdaController_testIfRunning");
+         nvdaController_cancelSpeech_func   = (unsigned long(__stdcall *)(void))dylib_proc(nvda_lib, "nvdaController_cancelSpeech");
+         nvdaController_brailleMessage_func = (unsigned long(__stdcall *)(wchar_t*))dylib_proc(nvda_lib, "nvdaController_brailleMessage");
+         nvdaController_speakText_func      = (unsigned long(__stdcall *)(wchar_t*))dylib_proc(nvda_lib, "nvdaController_speakText");
+         return;
       }
    }
-#else
-   USE_NVDA = false;
-   USE_POWERSHELL = true;
 #endif
+   /* The above code is executed on each accessibility speak event, so
+    * we should only revert to powershell if nvda_lib wasn't loaded previously,
+    * and we weren't able to load it on this call, or we don't HAVE_DYLIB */
+   if ((g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA) && !nvda_lib)
+   {
+      g_plat_win32_flags &= ~PLAT_WIN32_FLAG_USE_NVDA;
+      g_plat_win32_flags |=  PLAT_WIN32_FLAG_USE_POWERSHELL;
+   }
 }
 #endif
 
@@ -554,11 +549,11 @@ static int frontend_win32_parse_drive_list(void *data, bool load_content)
    {
       drive[0] = 'A' + i;
       if (drives & (1 << i))
-         menu_entries_append_enum(list,
+         menu_entries_append(list,
                drive,
                msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
                enum_idx,
-               FILE_TYPE_DIRECTORY, 0, 0);
+               FILE_TYPE_DIRECTORY, 0, 0, NULL);
    }
 #endif
 
@@ -569,6 +564,7 @@ static void frontend_win32_env_get(int *argc, char *argv[],
       void *args, void *params_data)
 {
    const char *tmp_dir = getenv("TMP");
+   const char *libretro_directory = getenv("LIBRETRO_DIRECTORY");
    if (!string_is_empty(tmp_dir))
       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CACHE],
          tmp_dir, sizeof(g_defaults.dirs[DEFAULT_DIR_CACHE]));
@@ -585,8 +581,6 @@ static void frontend_win32_env_get(int *argc, char *argv[],
       ":\\cheats", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_DATABASE],
       ":\\database\\rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CURSOR],
-      ":\\database\\cursors", sizeof(g_defaults.dirs[DEFAULT_DIR_CURSOR]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_PLAYLIST],
       ":\\playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG],
@@ -603,12 +597,12 @@ static void frontend_win32_env_get(int *argc, char *argv[],
       ":\\thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_OVERLAY],
       ":\\overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
-#ifdef HAVE_VIDEO_LAYOUT
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT],
-      ":\\layouts", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_LAYOUT]));
-#endif
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE],
-      ":\\cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
+   if (!string_is_empty(libretro_directory))
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_CORE], libretro_directory,
+            sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
+   else
+      fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE],
+            ":\\cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE_INFO],
       ":\\info", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG],
@@ -671,12 +665,12 @@ static void frontend_win32_attach_console(void)
 {
 #ifdef _WIN32
 #ifdef _WIN32_WINNT_WINXP
-   /* msys will start the process with FILE_TYPE_PIPE connected.
-    *   cmd will start the process with FILE_TYPE_UNKNOWN connected
+   /* MSys will start the process with FILE_TYPE_PIPE connected.
+    * cmd will start the process with FILE_TYPE_UNKNOWN connected
     *   (since this is subsystem windows application
     * ... UNLESS stdout/stderr were redirected (then FILE_TYPE_DISK
     * will be connected most likely)
-    * explorer will start the process with NOTHING connected.
+    * Explorer will start the process with NOTHING connected.
     *
     * Now, let's not reconnect anything that's already connected.
     * If any are disconnected, open a console, and connect to them.
@@ -692,21 +686,23 @@ static void frontend_win32_attach_console(void)
    bool need_stderr = (GetFileType(GetStdHandle(STD_ERROR_HANDLE))
          == FILE_TYPE_UNKNOWN);
 
+   if (config_get_ptr()->bools.log_to_file)
+      return;
+
    if (need_stdout || need_stderr)
    {
-      if (!AttachConsole( ATTACH_PARENT_PROCESS))
+      if (!AttachConsole(ATTACH_PARENT_PROCESS))
          AllocConsole();
 
       SetConsoleTitle("Log Console");
 
       if (need_stdout)
-         freopen( "CONOUT$", "w", stdout );
+         freopen("CONOUT$", "w", stdout);
       if (need_stderr)
-         freopen( "CONOUT$", "w", stderr );
+         freopen("CONOUT$", "w", stderr);
 
-      console_needs_free = true;
+      g_plat_win32_flags |= PLAT_WIN32_FLAG_CONSOLE_NEEDS_FREE;
    }
-
 #endif
 #endif
 }
@@ -715,13 +711,13 @@ static void frontend_win32_detach_console(void)
 {
 #if defined(_WIN32) && !defined(_XBOX)
 #ifdef _WIN32_WINNT_WINXP
-   if (console_needs_free)
+   if (g_plat_win32_flags & PLAT_WIN32_FLAG_CONSOLE_NEEDS_FREE)
    {
-      /* we don't reconnect stdout/stderr to anything here,
+      /* We don't reconnect stdout/stderr to anything here,
        * because by definition, they weren't connected to
        * anything in the first place. */
       FreeConsole();
-      console_needs_free = false;
+      g_plat_win32_flags &= ~PLAT_WIN32_FLAG_CONSOLE_NEEDS_FREE;
    }
 #endif
 #endif
@@ -729,12 +725,8 @@ static void frontend_win32_detach_console(void)
 
 static const char* frontend_win32_get_cpu_model_name(void)
 {
-#ifdef ANDROID
-   return NULL;
-#else
    cpu_features_get_model_name(win32_cpu_model_name, sizeof(win32_cpu_model_name));
    return win32_cpu_model_name;
-#endif
 }
 
 enum retro_language frontend_win32_get_user_language(void)
@@ -757,6 +749,7 @@ static void frontend_win32_respawn(char *s, size_t len, char *args)
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    char executable_path[PATH_MAX_LENGTH] = {0};
+   char executable_args[PATH_MAX_LENGTH] = {0};
 
    if (win32_fork_mode != FRONTEND_FORK_RESTART)
       return;
@@ -764,18 +757,17 @@ static void frontend_win32_respawn(char *s, size_t len, char *args)
    fill_pathname_application_path(executable_path,
          sizeof(executable_path));
    path_set(RARCH_PATH_CORE, executable_path);
-   RARCH_LOG("Restarting RetroArch with commandline: %s and %s\n",
-      executable_path, args);
+
+   /* Remove executable path from arguments given to CreateProcess */
+   snprintf(executable_args, sizeof(executable_args), "%s", strstr(args, ".exe") + 4);
 
    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    memset(&pi, 0, sizeof(pi));
 
-   if (!CreateProcess( executable_path, args,
-      NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
-   {
-      RARCH_LOG("Failed to restart RetroArch\n");
-   }
+   if (!CreateProcess(executable_path, executable_args,
+         NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
+      RARCH_ERR("Failed to restart RetroArch\n");
 }
 
 static bool frontend_win32_set_fork(enum frontend_fork fork_mode)
@@ -861,10 +853,7 @@ static const char *accessibility_win_language_id(const char* language)
       return "415";
    else if (string_is_equal(language,"cs")) 
       return "405";
-   else
-      return "";
-
-
+   return "";
 }
 
 static const char *accessibility_win_language_code(const char* language)
@@ -929,16 +918,14 @@ static const char *accessibility_win_language_code(const char* language)
       return "Microsoft Adam Desktop";
    else if (string_is_equal(language,"cs")) 
       return "Microsoft Jakub Desktop";
-   else
-      return "";
+   return "";
 }
 
-static bool terminate_win32_process(PROCESS_INFORMATION pi)
+static void terminate_win32_process(PROCESS_INFORMATION pi)
 {
    TerminateProcess(pi.hProcess,0);
    CloseHandle(pi.hProcess);
    CloseHandle(pi.hThread);
-   return true;
 }
 
 static PROCESS_INFORMATION g_pi;
@@ -947,7 +934,7 @@ static bool create_win32_process(char* cmd, const char * input)
 {
    STARTUPINFO si;
    HANDLE rd = NULL;
-   bool ret;
+   bool ret  = false;
    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    memset(&g_pi, 0, sizeof(g_pi));
@@ -956,22 +943,25 @@ static bool create_win32_process(char* cmd, const char * input)
    {
       DWORD dummy;
       HANDLE wr;
-      if (!CreatePipe(&rd, &wr, NULL, strlen(input))) return false;
+      size_t input_len = strlen(input);
+      if (!CreatePipe(&rd, &wr, NULL, input_len))
+         return false;
       
       SetHandleInformation(rd, HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT);
       
-      WriteFile(wr, input, strlen(input), &dummy, NULL);
+      WriteFile(wr, input, input_len, &dummy, NULL);
       CloseHandle(wr);
       
-      si.dwFlags |= STARTF_USESTDHANDLES;
-      si.hStdInput = rd;
-      si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
-      si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
+      si.dwFlags    |= STARTF_USESTDHANDLES;
+      si.hStdInput   = rd;
+      si.hStdOutput  = GetStdHandle(STD_OUTPUT_HANDLE);
+      si.hStdError   = GetStdHandle(STD_ERROR_HANDLE);
    }
 
    ret = CreateProcess(NULL, cmd, NULL, NULL, TRUE, CREATE_NO_WINDOW,
                       NULL, NULL, &si, &g_pi);
-   if (rd) CloseHandle(rd);
+   if (rd)
+      CloseHandle(rd);
    return ret;
 }
 
@@ -982,32 +972,29 @@ static bool is_narrator_running_windows(void)
    init_nvda();
 #endif
 
-   if (USE_POWERSHELL)
+   if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_POWERSHELL)
    {
-      if (pi_set == false)
+      if (!(g_plat_win32_flags & PLAT_WIN32_FLAG_PROCESS_INSTANCE_SET))
          return false;
       if (GetExitCodeProcess(g_pi.hProcess, &status))
-      {
          if (status == STILL_ACTIVE)
             return true;
-      }
       return false;
    }
 #ifdef HAVE_NVDA
-   else if (USE_NVDA)
+   else if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA)
    {
-      long res;
-      res = nvdaController_testIfRunning_func();
+      long res = nvdaController_testIfRunning_func();
 
       if (res != 0) 
       {
          /* The running nvda service wasn't found, so revert
             back to the powershell method
          */
-         RARCH_LOG("Error communicating with NVDA\n");
-         USE_POWERSHELL = true;
-         USE_NVDA       = false;
-	 return false;
+         RARCH_ERR("Error communicating with NVDA\n");
+         g_plat_win32_flags |=  PLAT_WIN32_FLAG_USE_POWERSHELL;
+         g_plat_win32_flags &= ~PLAT_WIN32_FLAG_USE_NVDA;
+         return false;
       }
       return false;
    }
@@ -1015,11 +1002,11 @@ static bool is_narrator_running_windows(void)
 #ifdef HAVE_SAPI
    else
    {
-      SPVOICESTATUS pStatus;
-      if (pVoice)
+      if (voice_ptr)
       {
-         ISpVoice_GetStatus(pVoice, &pStatus, NULL);
-         if (pStatus.dwRunningState == SPRS_IS_SPEAKING)
+         SPVOICESTATUS status_ptr;
+         ISpVoice_GetStatus(voice_ptr, &status_ptr, NULL);
+         if (status_ptr.dwRunningState == SPRS_IS_SPEAKING)
             return true;
       }
    }
@@ -1036,44 +1023,45 @@ static bool accessibility_speak_windows(int speed,
    const char *langid     = accessibility_win_language_id(voice);
    bool res               = false;
    const char* speeds[10] = {"-10", "-7.5", "-5", "-2.5", "0", "2", "4", "6", "8", "10"};
-   size_t nbytes_cmd = 0;
+   size_t nbytes_cmd      = 0;
    if (speed < 1)
-      speed = 1;
+      speed               = 1;
    else if (speed > 10)
-      speed = 10;
+      speed               = 10;
 
    if (priority < 10)
    {
       if (is_narrator_running_windows())
          return true;
-   
    }
 #ifdef HAVE_NVDA
    init_nvda();
 #endif
    
-   if (USE_POWERSHELL)
+   if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_POWERSHELL)
    {
       const char * template_lang = "powershell.exe -NoProfile -WindowStyle Hidden -Command \"Add-Type -AssemblyName System.Speech; $synth = New-Object System.Speech.Synthesis.SpeechSynthesizer; $synth.SelectVoice(\\\"%s\\\"); $synth.Rate = %s; $synth.Speak($input);\"";
       const char * template_nolang = "powershell.exe -NoProfile -WindowStyle Hidden -Command \"Add-Type -AssemblyName System.Speech; $synth = New-Object System.Speech.Synthesis.SpeechSynthesizer; $synth.Rate = %s; $synth.Speak($input);\"";
-      if (strlen(language) > 0)
+      if (language && language[0] != '\0')
          snprintf(cmd, sizeof(cmd), template_lang, language, speeds[speed-1]);
       else
          snprintf(cmd, sizeof(cmd), template_nolang, speeds[speed-1]);
-
-      if (pi_set)
+      if (g_plat_win32_flags & PLAT_WIN32_FLAG_PROCESS_INSTANCE_SET)
          terminate_win32_process(g_pi);
-      pi_set = create_win32_process(cmd, speak_text);
+      if (create_win32_process(cmd, speak_text))
+         g_plat_win32_flags |=  PLAT_WIN32_FLAG_PROCESS_INSTANCE_SET;
+      else
+         g_plat_win32_flags &= ~PLAT_WIN32_FLAG_PROCESS_INSTANCE_SET;
    }
 #ifdef HAVE_NVDA
-   else if (USE_NVDA)
+   else if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA)
    {
       wchar_t        *wc = utf8_to_utf16_string_alloc(speak_text);
       long res           = nvdaController_testIfRunning_func();
 
       if (!wc || res != 0) 
       {
-         RARCH_LOG("Error communicating with NVDA\n");
+         RARCH_ERR("Error communicating with NVDA\n");
          if (wc)
             free(wc);
          return false;
@@ -1081,7 +1069,7 @@ static bool accessibility_speak_windows(int speed,
 
       nvdaController_cancelSpeech_func();
 
-      if (USE_NVDA_BRAILLE)
+      if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA_BRAILLE)
          nvdaController_brailleMessage_func(wc);
       else
          nvdaController_speakText_func(wc);
@@ -1093,26 +1081,26 @@ static bool accessibility_speak_windows(int speed,
    {
       HRESULT hr;
       /* stop the old voice if running */
-      if (pVoice)
+      if (voice_ptr)
       {
          CoUninitialize();
-         ISpVoice_Release(pVoice);
+         ISpVoice_Release(voice_ptr);
       }
-      pVoice = NULL;
+      voice_ptr = NULL;
 
       /* Play the new voice */
       if (FAILED(CoInitialize(NULL)))
          return NULL;
 
       hr = CoCreateInstance(&CLSID_SpVoice, NULL,
-            CLSCTX_ALL, &IID_ISpVoice, (void **)&pVoice);
+            CLSCTX_ALL, &IID_ISpVoice, (void **)&voice_ptr);
 
       if (SUCCEEDED(hr))
       {
-         wchar_t        *wc = utf8_to_utf16_string_alloc(speak_text);
+         wchar_t *wc = utf8_to_utf16_string_alloc(speak_text);
          if (!wc)
             return false;
-         hr = ISpVoice_Speak(pVoice, wc, SPF_ASYNC /*SVSFlagsAsync*/, NULL);
+         hr = ISpVoice_Speak(voice_ptr, wc, SPF_ASYNC /*SVSFlagsAsync*/, NULL);
          free(wc);
       }
    }
@@ -1168,6 +1156,7 @@ frontend_ctx_driver_t frontend_ctx_win32 = {
    NULL,                            /* is_narrator_running */
    NULL,                            /* accessibility_speak */
 #endif
+   NULL,                            /* set_gamemode        */
    "win32",                         /* ident               */
    NULL                             /* get_video_driver    */
 };
