@@ -15,10 +15,14 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* OpenGL driver.
+/* OpenGL 1.x driver. 
  *
- * We are targeting a minimum of OpenGL 1.1 and the Microsoft "GDI Generic" software GL implementation.
- * Any additional features added for later 1.x versions should only be enabled if they are detected at runtime. */
+ * Minimum version : OpenGL 1.1 (1997)
+ *
+ * We are targeting a minimum of OpenGL 1.1 and the Microsoft 
+ * "GDI Generic" * software GL implementation.
+ * Any additional features added for later 1.x versions should only be 
+ * enabled if they are detected at runtime. */
 
 #include <stddef.h>
 #include <retro_miscellaneous.h>
@@ -41,7 +45,6 @@
 
 #include "../font_driver.h"
 
-#include "../../driver.h"
 #include "../../configuration.h"
 #include "../../retroarch.h"
 #include "../../verbosity.h"
@@ -92,20 +95,16 @@ static const GLfloat gl1_white_color[] = {
    1, 1, 1, 1,
 };
 
-#define gl1_context_bind_hw_render(gl1, enable) \
-   if (gl1->shared_context_use) \
-      gl1->ctx_driver->bind_hw_render(gl1->ctx_data, enable)
-
 #ifdef HAVE_OVERLAY
 static void gl1_render_overlay(gl1_t *gl,
       unsigned width,
       unsigned height)
 {
-   unsigned i;
+   int i;
 
    glEnable(GL_BLEND);
 
-   if (gl->overlay_full_screen)
+   if (gl->flags & GL1_FLAG_OVERLAY_FULLSCREEN)
       glViewport(0, 0, width, height);
 
    gl->coords.vertex    = gl->overlay_vertex_coord;
@@ -117,7 +116,7 @@ static void gl1_render_overlay(gl1_t *gl,
    glPushMatrix();
    glLoadIdentity();
 
-   for (i = 0; i < gl->overlays; i++)
+   for (i = 0; i < (int)gl->overlays; i++)
    {
       glBindTexture(GL_TEXTURE_2D, gl->overlay_tex[i]);
       glDrawArrays(GL_TRIANGLE_STRIP, 4 * i, 4);
@@ -128,7 +127,7 @@ static void gl1_render_overlay(gl1_t *gl,
    gl->coords.tex_coord = gl->tex_info.coord;
    gl->coords.color     = gl->white_color_ptr;
    gl->coords.vertices  = 4;
-   if (gl->overlay_full_screen)
+   if (gl->flags & GL1_FLAG_OVERLAY_FULLSCREEN)
       glViewport(gl->vp.x, gl->vp.y, gl->vp.width, gl->vp.height);
 }
 
@@ -205,15 +204,9 @@ static void gl1_overlay_tex_geom(void *data,
 
 #endif
 
-static bool is_pot(unsigned x)
-{
-   return (x & (x - 1)) == 0;
-}
+#define IS_POT(x) (((x) & (x - 1)) == 0)
 
-static unsigned get_pot(unsigned x)
-{
-   return (is_pot(x) ? x : next_pow2(x));
-}
+#define GET_POT(x) (IS_POT((x)) ? (x) : next_pow2((x)))
 
 static void *gl1_gfx_init(const video_info_t *video,
       input_driver_t **input, void **input_data)
@@ -229,13 +222,13 @@ static void *gl1_gfx_init(const video_info_t *video,
    bool video_smooth                    = settings->bools.video_smooth;
    bool video_font_enable               = settings->bools.video_font_enable;
    const char *video_context_driver     = settings->arrays.video_context_driver;
-   gl1_t *gl1                           = (gl1_t*)calloc(1, sizeof(*gl1));
    const char *vendor                   = NULL;
    const char *renderer                 = NULL;
    const char *version                  = NULL;
    const char *extensions               = NULL;
    int interval                         = 0;
    struct retro_hw_render_callback *hwr = NULL;
+   gl1_t *gl1                           = (gl1_t*)calloc(1, sizeof(*gl1));
 
    if (!gl1)
       return NULL;
@@ -245,14 +238,18 @@ static void *gl1_gfx_init(const video_info_t *video,
 
    gl1->video_width                     = video->width;
    gl1->video_height                    = video->height;
-   gl1->rgb32                           = video->rgb32;
-
-   gl1->video_bits                      = video->rgb32 ? 32 : 16;
 
    if (video->rgb32)
+   {
+      gl1->video_bits                   = 32;
       gl1->video_pitch                  = video->width * 4;
+      gl1->flags                       |= GL1_FLAG_RGB32;
+   }
    else
+   {
+      gl1->video_bits                   = 16;
       gl1->video_pitch                  = video->width * 2;
+   }
 
    ctx_driver = video_context_driver_init_first(gl1,
          video_context_driver,
@@ -268,12 +265,21 @@ static void *gl1_gfx_init(const video_info_t *video,
 
    video_context_driver_set((const gfx_ctx_driver_t*)ctx_driver);
 
-   RARCH_LOG("[GL1]: Found GL1 context: %s\n", ctx_driver->ident);
+   RARCH_LOG("[GL1]: Found GL1 context: \"%s\".\n", ctx_driver->ident);
 
    if (gl1->ctx_driver->get_video_size)
       gl1->ctx_driver->get_video_size(gl1->ctx_data,
                &mode_width, &mode_height);
 
+#if defined(__APPLE__) && !defined(IOS)
+   /* This is a hack for now to work around a very annoying
+    * issue that currently eludes us. */
+   if (     !gl1->ctx_driver->set_video_mode
+         || !gl1->ctx_driver->set_video_mode(gl1->ctx_data,
+            win_width, win_height, video->fullscreen))
+      goto error;
+#endif
+
    full_x      = mode_width;
    full_y      = mode_height;
    mode_width  = 0;
@@ -292,28 +298,27 @@ static void *gl1_gfx_init(const video_info_t *video,
    if (string_is_equal(ctx_driver->ident, "null"))
       goto error;
 
-   RARCH_LOG("[GL1]: Detecting screen resolution %ux%u.\n", full_x, full_y);
-
-   win_width   = video->width;
-   win_height  = video->height;
+   RARCH_LOG("[GL1]: Detecting screen resolution: %ux%u.\n", full_x, full_y);
+   win_width       = video->width;
+   win_height      = video->height;
 
    if (video->fullscreen && (win_width == 0) && (win_height == 0))
    {
-      win_width  = full_x;
-      win_height = full_y;
+      win_width    = full_x;
+      win_height   = full_y;
    }
 
    mode_width      = win_width;
    mode_height     = win_height;
 
-   interval = video->swap_interval;
+   interval        = video->swap_interval;
 
    if (ctx_driver->swap_interval)
    {
-      bool adaptive_vsync_enabled            = video_driver_test_all_flags(
+      bool adaptive_vsync_enabled = video_driver_test_all_flags(
             GFX_CTX_FLAGS_ADAPTIVE_VSYNC) && video->adaptive_vsync;
       if (adaptive_vsync_enabled && interval == 1)
-         interval = -1;
+         interval                 = -1;
       ctx_driver->swap_interval(gl1->ctx_data, interval);
    }
 
@@ -322,7 +327,8 @@ static void *gl1_gfx_init(const video_info_t *video,
             win_width, win_height, video->fullscreen))
       goto error;
 
-   gl1->fullscreen = video->fullscreen;
+   if (video->fullscreen)
+      gl1->flags |= GL1_FLAG_FULLSCREEN;
 
    mode_width     = 0;
    mode_height    = 0;
@@ -341,7 +347,7 @@ static void *gl1_gfx_init(const video_info_t *video,
 
    video_driver_get_size(&temp_width, &temp_height);
 
-   RARCH_LOG("[GL1]: Using resolution %ux%u\n", temp_width, temp_height);
+   RARCH_LOG("[GL1]: Using resolution %ux%u.\n", temp_width, temp_height);
 
    vendor   = (const char*)glGetString(GL_VENDOR);
    renderer = (const char*)glGetString(GL_RENDERER);
@@ -365,8 +371,9 @@ static void *gl1_gfx_init(const video_info_t *video,
 
       if (!string_is_empty(vendor))
       {
-         strlcpy(device_str, vendor, sizeof(device_str));
-         strlcat(device_str, " ", sizeof(device_str));
+         size_t len        = strlcpy(device_str, vendor, sizeof(device_str));
+         device_str[len  ] = ' ';
+         device_str[len+1] = '\0';
       }
 
       if (!string_is_empty(renderer))
@@ -393,8 +400,10 @@ static void *gl1_gfx_init(const video_info_t *video,
             video->is_threaded,
             FONT_DRIVER_RENDER_OPENGL1_API);
 
-   gl1->smooth        = video_smooth;
-   gl1->supports_bgra = string_list_find_elem(gl1->extensions, "GL_EXT_bgra");
+   if (video_smooth)
+      gl1->flags     |= GL1_FLAG_SMOOTH;
+   if (string_list_find_elem(gl1->extensions, "GL_EXT_bgra"))
+      gl1->flags     |= GL1_FLAG_SUPPORTS_BGRA;
 
    glDisable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
@@ -410,8 +419,9 @@ static void *gl1_gfx_init(const video_info_t *video,
    hwr = video_driver_get_hw_context();
 
    memcpy(gl1->tex_info.coord, gl1_tex_coords, sizeof(gl1->tex_info.coord));
-   gl1->vertex_ptr        = hwr->bottom_left_origin
-      ? gl1_vertexes : gl1_vertexes_flipped;
+   gl1->vertex_ptr            = hwr->bottom_left_origin
+                              ? gl1_vertexes 
+                              : gl1_vertexes_flipped;
    gl1->textures              = 4;
    gl1->white_color_ptr       = gl1_white_color;
    gl1->coords.vertex         = gl1->vertex_ptr;
@@ -420,12 +430,10 @@ static void *gl1_gfx_init(const video_info_t *video,
    gl1->coords.lut_tex_coord  = gl1_tex_coords;
    gl1->coords.vertices       = 4;
 
-   RARCH_LOG("[GL1]: Init complete.\n");
-
    return gl1;
 
 error:
-   video_context_driver_destroy();
+   video_context_driver_free();
    if (gl1)
    {
       if (gl1->extensions)
@@ -438,7 +446,13 @@ error:
 static void gl1_set_projection(gl1_t *gl1,
       struct video_ortho *ortho, bool allow_rotate)
 {
-   math_matrix_4x4 rot;
+   static math_matrix_4x4 rot     = {
+      { 0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    1.0f }
+   };
+   float radians, cosine, sine;
 
    /* Calculate projection. */
    matrix_4x4_ortho(gl1->mvp_no_rot, ortho->left, ortho->right,
@@ -450,7 +464,13 @@ static void gl1_set_projection(gl1_t *gl1,
       return;
    }
 
-   matrix_4x4_rotate_z(rot, M_PI * gl1->rotation / 180.0f);
+   radians                 = M_PI * gl1->rotation / 180.0f;
+   cosine                  = cosf(radians);
+   sine                    = sinf(radians);
+   MAT_ELEM_4X4(rot, 0, 0) = cosine;
+   MAT_ELEM_4X4(rot, 0, 1) = -sine;
+   MAT_ELEM_4X4(rot, 1, 0) = sine;
+   MAT_ELEM_4X4(rot, 1, 1) = cosine;
    matrix_4x4_multiply(gl1->mvp, rot, gl1->mvp_no_rot);
 }
 
@@ -473,11 +493,12 @@ void gl1_gfx_set_viewport(gl1_t *gl1,
    {
       video_viewport_get_scaled_integer(&gl1->vp,
             viewport_width, viewport_height,
-            video_driver_get_aspect_ratio(), gl1->keep_aspect);
+            video_driver_get_aspect_ratio(),
+            gl1->flags & GL1_FLAG_KEEP_ASPECT);
       viewport_width  = gl1->vp.width;
       viewport_height = gl1->vp.height;
    }
-   else if (gl1->keep_aspect && !force_full)
+   else if ((gl1->flags & GL1_FLAG_KEEP_ASPECT) && !force_full)
    {
       float desired_aspect = video_driver_get_aspect_ratio();
 
@@ -550,33 +571,37 @@ void gl1_gfx_set_viewport(gl1_t *gl1,
 #endif
 }
 
-static void draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int height, GLuint tex, const void *frame_to_copy)
+static void gl1_gfx_draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int height, GLuint tex, const void *frame_to_copy)
 {
-   uint8_t *frame       = NULL;
-   uint8_t *frame_rgba  = NULL;
+   uint8_t *frame         = NULL;
+   uint8_t *frame_rgba    = NULL;
    /* FIXME: For now, everything is uploaded as BGRA8888, I could not get 444 or 555 to work, and there is no 565 support in GL 1.1 either. */
-   GLint internalFormat = GL_RGB8;
-   GLenum format        = gl1->supports_bgra ? GL_BGRA_EXT : GL_RGBA;
-   GLenum type          = GL_UNSIGNED_BYTE;
-
-   float vertices[] = {
+   GLint internalFormat   = GL_RGB8;
+   bool   supports_native = gl1->flags & GL1_FLAG_SUPPORTS_BGRA;
+   GLenum format          = supports_native ? GL_BGRA_EXT : GL_RGBA;
+#ifdef MSB_FIRST
+   GLenum type            = supports_native ? GL_UNSIGNED_INT_8_8_8_8_REV : GL_UNSIGNED_BYTE;
+#else
+   GLenum type            = GL_UNSIGNED_BYTE;
+#endif
+   float vertices[]       = {
 	   -1.0f, -1.0f, 0.0f,
-	   -1.0f, 1.0f, 0.0f,
-	   1.0f, -1.0f, 0.0f,
-	   1.0f, 1.0f, 0.0f,
+	   -1.0f,  1.0f, 0.0f,
+	    1.0f, -1.0f, 0.0f,
+	    1.0f,  1.0f, 0.0f,
    };
 
-   float colors[] = {
+   float colors[]         = {
       1.0f, 1.0f, 1.0f, 1.0f,
       1.0f, 1.0f, 1.0f, 1.0f,
       1.0f, 1.0f, 1.0f, 1.0f,
       1.0f, 1.0f, 1.0f, 1.0f
    };
 
-   float norm_width     = (1.0f / (float)pot_width) * (float)width;
-   float norm_height    = (1.0f / (float)pot_height) * (float)height;
+   float norm_width       = (1.0f / (float)pot_width) * (float)width;
+   float norm_height      = (1.0f / (float)pot_height) * (float)height;
    
-   float texcoords[] = {
+   float texcoords[]      = {
       0.0f, 0.0f,
       0.0f, 0.0f,
       0.0f, 0.0f,
@@ -602,7 +627,8 @@ static void draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int h
    glBindTexture(GL_TEXTURE_2D, tex);
 
    frame = (uint8_t*)frame_to_copy;
-   if (!gl1->supports_bgra)
+
+   if (!supports_native)
    {
       frame_rgba = (uint8_t*)malloc(pot_width * pot_height * 4);
       if (frame_rgba)
@@ -613,10 +639,17 @@ static void draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int h
             for (x = 0; x < pot_width; x++)
             {
                int index             = (y * pot_width + x) * 4;
+#ifdef MSB_FIRST
+               frame_rgba[index + 2] = frame[index + 3];
+               frame_rgba[index + 1] = frame[index + 2];
+               frame_rgba[index + 0] = frame[index + 1];
+               frame_rgba[index + 3] = frame[index + 0];
+#else
                frame_rgba[index + 2] = frame[index + 0];
                frame_rgba[index + 1] = frame[index + 1];
                frame_rgba[index + 0] = frame[index + 2];
                frame_rgba[index + 3] = frame[index + 3];
+#endif
             }
          }
          frame = frame_rgba;
@@ -629,13 +662,29 @@ static void draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int h
 
    if (tex == gl1->tex)
    {
-      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (gl1->smooth ? GL_LINEAR : GL_NEAREST));
-      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (gl1->smooth ? GL_LINEAR : GL_NEAREST));
+      if (gl1->flags & GL1_FLAG_SMOOTH)
+      {
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+      }
+      else
+      {
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+      }
    }
    else if (tex == gl1->menu_tex)
    {
-      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (gl1->menu_smooth ? GL_LINEAR : GL_NEAREST));
-      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (gl1->menu_smooth ? GL_LINEAR : GL_NEAREST));
+      if (gl1->flags & GL1_FLAG_MENU_SMOOTH)
+      {
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+      }
+      else
+      {
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+      }
    }
 
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
@@ -672,10 +721,8 @@ static void draw_tex(gl1_t *gl1, int pot_width, int pot_height, int width, int h
    glPopMatrix();
 }
 
-static void gl1_readback(
-      gl1_t *gl1,
-      unsigned alignment,
-      unsigned fmt, unsigned type,
+static void gl1_readback(gl1_t *gl1,
+      unsigned alignment, unsigned fmt, unsigned type,
       void *src)
 {
 #ifndef VITA
@@ -714,18 +761,17 @@ static bool gl1_gfx_frame(void *data, const void *frame,
    bool hard_sync                   = video_info->hard_sync;
    struct font_params *osd_params   = (struct font_params*)
       &video_info->osd_stat_params;
+   bool overlay_behind_menu         = video_info->overlay_behind_menu;
 
    /* FIXME: Force these settings off as they interfere with the rendering */
    video_info->xmb_shadows_enable   = false;
    video_info->menu_shader_pipeline = 0;
 
-   gl1_context_bind_hw_render(gl1, false);
-
-   if (gl1->should_resize)
+   if (gl1->flags & GL1_FLAG_SHOULD_RESIZE)
    {
       gfx_ctx_mode_t mode;
 
-      gl1->should_resize = false;
+      gl1->flags       &= ~GL1_FLAG_SHOULD_RESIZE;
 
       mode.width        = width;
       mode.height       = height;
@@ -738,18 +784,20 @@ static bool gl1_gfx_frame(void *data, const void *frame,
             video_width, video_height, false, true);
    }
 
-   if (  !frame || frame == RETRO_HW_FRAME_BUFFER_VALID || (
-         frame_width  == 4 &&
-         frame_height == 4 &&
-         (frame_width < width && frame_height < height))
+   if (     !frame
+         || (frame == RETRO_HW_FRAME_BUFFER_VALID)
+         || (
+            (frame_width  == 4)
+         && (frame_height == 4)
+         && (frame_width < width && frame_height < height))
       )
       draw = false;
    
    do_swap = frame || draw;
 
-   if (  gl1->video_width  != frame_width  ||
-         gl1->video_height != frame_height ||
-         gl1->video_pitch  != pitch)
+   if (     (gl1->video_width  != frame_width)
+         || (gl1->video_height != frame_height)
+         || (gl1->video_pitch  != pitch))
    {
       if (frame_width > 4 && frame_height > 4)
       {
@@ -757,8 +805,8 @@ static bool gl1_gfx_frame(void *data, const void *frame,
          gl1->video_height = frame_height;
          gl1->video_pitch  = pitch;
 
-         pot_width = get_pot(frame_width);
-         pot_height = get_pot(frame_height);
+         pot_width         = GET_POT(frame_width);
+         pot_height        = GET_POT(frame_height);
          
          if (draw)
          {
@@ -774,17 +822,19 @@ static bool gl1_gfx_frame(void *data, const void *frame,
    height        = gl1->video_height;
    pitch         = gl1->video_pitch;
 
-   pot_width = get_pot(width);
-   pot_height = get_pot(height);
+   pot_width     = GET_POT(width);
+   pot_height    = GET_POT(height);
 
    if (draw && gl1->video_buf)
    {
       if (bits == 32)
       {
-         unsigned y;
+         int y;
          /* copy lines into top-left portion of larger (power-of-two) buffer */
-         for (y = 0; y < height; y++)
-            memcpy(gl1->video_buf + ((pot_width * (bits / 8)) * y), (const unsigned char*)frame + (pitch * y), width * (bits / 8));
+         for (y = 0; y < (int)height; y++)
+            memcpy(gl1->video_buf + ((pot_width * (bits / 8)) * y),
+                  (const unsigned char*)frame + (pitch * y),
+                  width * (bits / 8));
       }
       else if (bits == 16)
          conv_rgb565_argb8888(gl1->video_buf, frame, width, height, pot_width * sizeof(unsigned), pitch);
@@ -811,7 +861,7 @@ static bool gl1_gfx_frame(void *data, const void *frame,
       glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
       if (frame_to_copy)
-         draw_tex(gl1, pot_width, pot_height,
+         gl1_gfx_draw_tex(gl1, pot_width, pot_height,
                width, height, gl1->tex, frame_to_copy);
    }
 
@@ -824,14 +874,14 @@ static bool gl1_gfx_frame(void *data, const void *frame,
       pitch         = gl1->menu_pitch;
       bits          = gl1->menu_bits;
 
-      pot_width = get_pot(width);
-      pot_height = get_pot(height);
+      pot_width     = GET_POT(width);
+      pot_height    = GET_POT(height);
 
-      do_swap = true;
+      do_swap       = true;
 
-      if (gl1->menu_size_changed)
+      if (gl1->flags & GL1_FLAG_MENU_SIZE_CHANGED)
       {
-         gl1->menu_size_changed = false;
+         gl1->flags &= ~GL1_FLAG_MENU_SIZE_CHANGED;
 
          if (gl1->menu_video_buf)
             free(gl1->menu_video_buf);
@@ -850,30 +900,36 @@ static bool gl1_gfx_frame(void *data, const void *frame,
 
          frame_to_copy = gl1->menu_video_buf;
 
-         if (gl1->menu_texture_full_screen)
+         if (gl1->flags & GL1_FLAG_MENU_TEXTURE_FULLSCREEN)
          {
             glViewport(0, 0, video_width, video_height);
-            draw_tex(gl1, pot_width, pot_height,
+            gl1_gfx_draw_tex(gl1, pot_width, pot_height,
                   width, height, gl1->menu_tex, frame_to_copy);
             glViewport(gl1->vp.x, gl1->vp.y, gl1->vp.width, gl1->vp.height);
          }
          else
-            draw_tex(gl1, pot_width, pot_height,
+            gl1_gfx_draw_tex(gl1, pot_width, pot_height,
                   width, height, gl1->menu_tex, frame_to_copy);
       }
    }
 
-   if (gl1->menu_texture_enable){
+#ifdef HAVE_OVERLAY
+   if ((gl1->flags & GL1_FLAG_OVERLAY_ENABLE) && overlay_behind_menu)
+      gl1_render_overlay(gl1, video_width, video_height);
+#endif
+
+   if (gl1->flags & GL1_FLAG_MENU_TEXTURE_ENABLE)
+   {
       do_swap = true;
 #ifdef VITA
       glUseProgram(0);
       bool enabled = glIsEnabled(GL_DEPTH_TEST);
-      if(enabled)
+      if (enabled)
          glDisable(GL_DEPTH_TEST);
 #endif
       menu_driver_frame(menu_is_alive, video_info);
 #ifdef VITA
-      if(enabled)
+      if (enabled)
          glEnable(GL_DEPTH_TEST);
 #endif
    }
@@ -900,7 +956,7 @@ static bool gl1_gfx_frame(void *data, const void *frame,
 #endif
 
 #ifdef HAVE_OVERLAY
-   if (gl1->overlay_enable)
+   if ((gl1->flags & GL1_FLAG_OVERLAY_ENABLE) && !overlay_behind_menu)
       gl1_render_overlay(gl1, video_width, video_height);
 #endif
 
@@ -914,7 +970,13 @@ static bool gl1_gfx_frame(void *data, const void *frame,
    /* Screenshots. */
    if (gl1->readback_buffer_screenshot)
       gl1_readback(gl1,
-            4, GL_RGBA, GL_UNSIGNED_BYTE,
+            4,
+            GL_RGBA,
+#ifdef MSB_FIRST
+            GL_UNSIGNED_INT_8_8_8_8_REV,
+#else
+            GL_UNSIGNED_BYTE,
+#endif
             gl1->readback_buffer_screenshot);
 
 
@@ -930,11 +992,10 @@ static bool gl1_gfx_frame(void *data, const void *frame,
          && !video_info->input_driver_nonblock_state
          && !video_info->runloop_is_slowmotion
          && !video_info->runloop_is_paused 
-         && !gl1->menu_texture_enable)
+         && !(gl1->flags & GL1_FLAG_MENU_TEXTURE_ENABLE))
    {
-
-        unsigned n;
-        for (n = 0; n < video_info->black_frame_insertion; ++n)
+        int n;
+        for (n = 0; n < (int)video_info->black_frame_insertion; ++n)
         {
           glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
           glClear(GL_COLOR_BUFFER_BIT);			
@@ -945,22 +1006,22 @@ static bool gl1_gfx_frame(void *data, const void *frame,
    }   
 #endif 
 
-   /* check if we are fast forwarding or in menu, if we are ignore hard sync */
-   if (hard_sync
+   /* check if we are fast forwarding or in menu, 
+      if we are ignore hard sync */
+   if (      hard_sync
          && !video_info->input_driver_nonblock_state
-         && !gl1->menu_texture_enable)
+      )
    {
       glClear(GL_COLOR_BUFFER_BIT);
       glFinish();
    }
 
-   if(draw){
+   if (draw)
+   {
       glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
       glClear(GL_COLOR_BUFFER_BIT);
    }
 
-   gl1_context_bind_hw_render(gl1, true);
-
    return true;
 }
 
@@ -974,8 +1035,6 @@ static void gl1_gfx_set_nonblock_state(void *data, bool state,
    if (!gl1)
       return;
 
-   gl1_context_bind_hw_render(gl1, false);
-
    if (!state)
       interval = swap_interval;
 
@@ -985,7 +1044,6 @@ static void gl1_gfx_set_nonblock_state(void *data, bool state,
          interval = -1;
       gl1->ctx_driver->swap_interval(gl1->ctx_data, interval);
    }
-   gl1_context_bind_hw_render(gl1, true);
 }
 
 static bool gl1_gfx_alive(void *data)
@@ -1004,7 +1062,7 @@ static bool gl1_gfx_alive(void *data)
             &quit, &resize, &temp_width, &temp_height);
 
    if (resize)
-      gl1->should_resize = true;
+      gl1->flags        |= GL1_FLAG_SHOULD_RESIZE;
 
    ret = !quit;
 
@@ -1022,12 +1080,7 @@ static bool gl1_gfx_focus(void *data)
    return true;
 }
 
-static bool gl1_gfx_suppress_screensaver(void *data, bool enable)
-{
-   (void)data;
-   (void)enable;
-   return false;
-}
+static bool gl1_gfx_suppress_screensaver(void *data, bool enable) { return false; }
 
 static void gl1_gfx_free(void *data)
 {
@@ -1036,8 +1089,6 @@ static void gl1_gfx_free(void *data)
    if (!gl1)
       return;
 
-   gl1_context_bind_hw_render(gl1, false);
-
    if (gl1->menu_frame)
       free(gl1->menu_frame);
    gl1->menu_frame = NULL;
@@ -1078,14 +1129,7 @@ static void gl1_gfx_free(void *data)
 }
 
 static bool gl1_gfx_set_shader(void *data,
-      enum rarch_shader_type type, const char *path)
-{
-   (void)data;
-   (void)type;
-   (void)path;
-
-   return false;
-}
+      enum rarch_shader_type type, const char *path) { return false; }
 
 static void gl1_gfx_set_rotation(void *data,
       unsigned rotation)
@@ -1104,7 +1148,7 @@ static void gl1_gfx_viewport_info(void *data,
 {
    unsigned width, height;
    unsigned top_y, top_dist;
-   gl1_t *gl1             = (gl1_t*)data;
+   gl1_t *gl1      = (gl1_t*)data;
 
    video_driver_get_size(&width, &height);
 
@@ -1126,14 +1170,11 @@ static bool gl1_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
    if (!gl1)
       return false;
 
-   gl1_context_bind_hw_render(gl1, false);
-
-   num_pixels = gl1->vp.width * gl1->vp.height;
-
+   num_pixels                      = gl1->vp.width * gl1->vp.height;
    gl1->readback_buffer_screenshot = malloc(num_pixels * sizeof(uint32_t));
 
    if (!gl1->readback_buffer_screenshot)
-      goto error;
+      return false;
 
    if (!is_idle)
       video_driver_cached_frame();
@@ -1146,42 +1187,37 @@ static bool gl1_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
    free(gl1->readback_buffer_screenshot);
    gl1->readback_buffer_screenshot = NULL;
 
-   gl1_context_bind_hw_render(gl1, true);
    return true;
-
-error:
-   gl1_context_bind_hw_render(gl1, true);
-
-   return false;
 }
 
 static void gl1_set_texture_frame(void *data,
       const void *frame, bool rgb32, unsigned width, unsigned height,
       float alpha)
 {
-   settings_t *settings    = config_get_ptr();
-   bool menu_linear_filter = settings->bools.menu_linear_filter;
-   unsigned       pitch    = width * 2;
-   gl1_t              *gl1 = (gl1_t*)data;
+   settings_t *settings      = config_get_ptr();
+   bool menu_linear_filter   = settings->bools.menu_linear_filter;
+   unsigned       pitch      = width * 2;
+   gl1_t              *gl1   = (gl1_t*)data;
 
    if (!gl1)
       return;
 
-   gl1->menu_smooth        = menu_linear_filter;
-
-   gl1_context_bind_hw_render(gl1, false);
+   if (menu_linear_filter)
+      gl1->flags            |=  GL1_FLAG_MENU_SMOOTH;
+   else
+      gl1->flags            &= ~GL1_FLAG_MENU_SMOOTH;
 
    if (rgb32)
-      pitch = width * 4;
+      pitch                  = width * 4;
 
    if (gl1->menu_frame)
       free(gl1->menu_frame);
-   gl1->menu_frame = NULL;
+   gl1->menu_frame           = NULL;
 
-   if ( !gl1->menu_frame            ||
-         gl1->menu_width != width   ||
-         gl1->menu_height != height ||
-         gl1->menu_pitch != pitch)
+   if (     (!gl1->menu_frame)
+         || (gl1->menu_width  != width)
+         || (gl1->menu_height != height)
+         || (gl1->menu_pitch  != pitch))
    {
       if (pitch && height)
       {
@@ -1203,37 +1239,32 @@ static void gl1_set_texture_frame(void *data,
       gl1->menu_height       = height;
       gl1->menu_pitch        = pitch;
       gl1->menu_bits         = rgb32 ? 32 : 16;
-      gl1->menu_size_changed = true;
+      gl1->flags            |= GL1_FLAG_MENU_SIZE_CHANGED;
    }
-
-   gl1_context_bind_hw_render(gl1, true);
 }
 
 static void gl1_get_video_output_size(void *data,
-      unsigned *width, unsigned *height)
+      unsigned *width, unsigned *height, char *desc, size_t desc_len)
 {
    gl1_t *gl         = (gl1_t*)data;
-   if (!gl || !gl->ctx_driver || !gl->ctx_driver->get_video_output_size)
-      return;
-   gl->ctx_driver->get_video_output_size(
-         gl->ctx_data,
-         width, height);
+   if (gl && gl->ctx_driver && gl->ctx_driver->get_video_output_size)
+      gl->ctx_driver->get_video_output_size(
+            gl->ctx_data,
+            width, height, desc, desc_len);
 }
 
 static void gl1_get_video_output_prev(void *data)
 {
    gl1_t *gl         = (gl1_t*)data;
-   if (!gl || !gl->ctx_driver || !gl->ctx_driver->get_video_output_prev)
-      return;
-   gl->ctx_driver->get_video_output_prev(gl->ctx_data);
+   if (gl && gl->ctx_driver && gl->ctx_driver->get_video_output_prev)
+      gl->ctx_driver->get_video_output_prev(gl->ctx_data);
 }
 
 static void gl1_get_video_output_next(void *data)
 {
    gl1_t *gl         = (gl1_t*)data;
-   if (!gl || !gl->ctx_driver || !gl->ctx_driver->get_video_output_next)
-      return;
-   gl->ctx_driver->get_video_output_next(gl->ctx_data);
+   if (gl && gl->ctx_driver && gl->ctx_driver->get_video_output_next)
+      gl->ctx_driver->get_video_output_next(gl->ctx_data);
 }
 
 static void gl1_set_video_mode(void *data, unsigned width, unsigned height,
@@ -1247,16 +1278,10 @@ static void gl1_set_video_mode(void *data, unsigned width, unsigned height,
 
 static unsigned gl1_wrap_type_to_enum(enum gfx_wrap_type type)
 {
-   switch (type)
-   {
-      case RARCH_WRAP_REPEAT:
-      case RARCH_WRAP_MIRRORED_REPEAT: /* mirrored not actually supported */
-         return GL_REPEAT;
-      default:
-         return GL_CLAMP;
-   }
-
-   return 0;
+   /* Mirrored not actually supported */
+   if (type == RARCH_WRAP_REPEAT || type == RARCH_WRAP_MIRRORED_REPEAT)
+      return GL_REPEAT;
+   return GL_CLAMP;
 }
 
 static void gl1_load_texture_data(
@@ -1267,39 +1292,55 @@ static void gl1_load_texture_data(
       unsigned width, unsigned height,
       const void *frame, unsigned base_size)
 {
-   GLint mag_filter, min_filter;
+   GLint filter;
    bool use_rgba    = video_driver_supports_rgba();
    bool rgb32       = (base_size == (sizeof(uint32_t)));
    GLenum wrap      = gl1_wrap_type_to_enum(wrap_type);
 
+   glBindTexture(GL_TEXTURE_2D, id);
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap);
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap);
+
    /* GL1.x does not have mipmapping support. */
    switch (filter_type)
    {
       case TEXTURE_FILTER_MIPMAP_NEAREST:
       case TEXTURE_FILTER_NEAREST:
-         min_filter = GL_NEAREST;
-         mag_filter = GL_NEAREST;
+         filter = GL_NEAREST;
          break;
       case TEXTURE_FILTER_MIPMAP_LINEAR:
       case TEXTURE_FILTER_LINEAR:
       default:
-         min_filter = GL_LINEAR;
-         mag_filter = GL_LINEAR;
+         filter = GL_LINEAR;
          break;
    }
 
-   gl1_bind_texture(id, wrap, mag_filter, min_filter);
-
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
 #ifndef VITA
    glPixelStorei(GL_UNPACK_ALIGNMENT, alignment);
+   glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
 #endif
 
    glTexImage2D(GL_TEXTURE_2D,
          0,
-         (use_rgba || !rgb32) ? GL_RGBA : RARCH_GL1_INTERNAL_FORMAT32,
-         width, height, 0,
-         (use_rgba || !rgb32) ? GL_RGBA : RARCH_GL1_TEXTURE_TYPE32,
-         (rgb32) ? RARCH_GL1_FORMAT32 : GL_UNSIGNED_BYTE, frame);
+         (use_rgba || !rgb32) 
+         ? GL_RGBA 
+         : RARCH_GL1_INTERNAL_FORMAT32,
+         width,
+         height,
+         0,
+         (use_rgba || !rgb32) 
+         ? GL_RGBA 
+         : RARCH_GL1_TEXTURE_TYPE32,
+#ifdef MSB_FIRST
+         GL_UNSIGNED_INT_8_8_8_8_REV,
+#else
+         rgb32
+         ? RARCH_GL1_FORMAT32 
+         : GL_UNSIGNED_BYTE,
+#endif
+         frame);
 }
 
 static void video_texture_load_gl1(
@@ -1314,19 +1355,23 @@ static void video_texture_load_gl1(
 
    /* Generate the OpenGL texture object */
    glGenTextures(1, &id);
-   *idptr = id;
+   *idptr             = id;
 
    if (ti)
    {
-      width  = ti->width;
-      height = ti->height;
-      pixels = ti->pixels;
+      width           = ti->width;
+      height          = ti->height;
+      pixels          = ti->pixels;
    }
 
-   gl1_load_texture_data(id,
-         RARCH_WRAP_EDGE, filter_type,
+   gl1_load_texture_data(
+         id,
+         RARCH_WRAP_EDGE,
+         filter_type,
          4 /* TODO/FIXME - dehardcode */,
-         width, height, pixels,
+         width,
+         height,
+         pixels,
          sizeof(uint32_t) /* TODO/FIXME - dehardcode */
          );
 }
@@ -1335,7 +1380,6 @@ static void video_texture_load_gl1(
 static int video_texture_load_wrap_gl1(void *data)
 {
    uintptr_t id = 0;
-
    if (!data)
       return 0;
    video_texture_load_gl1((struct texture_image*)data,
@@ -1368,20 +1412,16 @@ static uintptr_t gl1_load_texture(void *video_data, void *data,
 
 static void gl1_set_aspect_ratio(void *data, unsigned aspect_ratio_idx)
 {
-   gl1_t *gl1         = (gl1_t*)data;
-
-   if (!gl1)
-      return;
-
-   gl1->keep_aspect   = true;
-   gl1->should_resize = true;
+   gl1_t *gl1     = (gl1_t*)data;
+   if (gl1)
+      gl1->flags |= (GL1_FLAG_KEEP_ASPECT | GL1_FLAG_SHOULD_RESIZE);
 }
 
 static void gl1_unload_texture(void *data, 
       bool threaded, uintptr_t id)
 {
    GLuint glid;
-   gl1_t               *gl1 = (gl1_t*)data;
+   gl1_t *gl1 = (gl1_t*)data;
    if (!id)
       return;
 
@@ -1407,22 +1447,29 @@ static float gl1_get_refresh_rate(void *data)
 
 static void gl1_set_texture_enable(void *data, bool state, bool full_screen)
 {
-   gl1_t *gl1                    = (gl1_t*)data;
+   gl1_t *gl1       = (gl1_t*)data;
 
    if (!gl1)
       return;
 
-   gl1->menu_texture_enable      = state;
-   gl1->menu_texture_full_screen = full_screen;
+   if (state)
+      gl1->flags   |=  GL1_FLAG_MENU_TEXTURE_ENABLE;
+   else
+      gl1->flags   &= ~GL1_FLAG_MENU_TEXTURE_ENABLE;
+   if (full_screen)
+      gl1->flags   |=  GL1_FLAG_MENU_TEXTURE_FULLSCREEN;
+   else
+      gl1->flags   &= ~GL1_FLAG_MENU_TEXTURE_FULLSCREEN;
 }
 
 static uint32_t gl1_get_flags(void *data)
 {
-   uint32_t             flags = 0;
+   uint32_t flags = 0;
 
    BIT32_SET(flags, GFX_CTX_FLAGS_HARD_SYNC);
    BIT32_SET(flags, GFX_CTX_FLAGS_BLACK_FRAME_INSERTION);
    BIT32_SET(flags, GFX_CTX_FLAGS_MENU_FRAME_FILTERING);
+   BIT32_SET(flags, GFX_CTX_FLAGS_OVERLAY_BEHIND_MENU_SUPPORTED);
 
    return flags;
 }
@@ -1448,22 +1495,17 @@ static const video_poke_interface_t gl1_poke_interface = {
    NULL,                         /* grab_mouse_toggle */
    NULL,                         /* get_current_shader */
    NULL,                         /* get_current_software_framebuffer */
-   NULL                          /* get_hw_render_interface */
+   NULL,                         /* get_hw_render_interface */
+   NULL,                         /* set_hdr_max_nits */
+   NULL,                         /* set_hdr_paper_white_nits */
+   NULL,                         /* set_hdr_contrast */
+   NULL                          /* set_hdr_expand_gamut */
 };
 
 static void gl1_gfx_get_poke_interface(void *data,
-      const video_poke_interface_t **iface)
-{
-   (void)data;
-   *iface = &gl1_poke_interface;
-}
-
+      const video_poke_interface_t **iface) { *iface = &gl1_poke_interface; }
 #ifdef HAVE_GFX_WIDGETS
-static bool gl1_gfx_widgets_enabled(void *data)
-{
-   (void)data;
-   return true;
-}
+static bool gl1_gfx_widgets_enabled(void *data) { return true; }
 #endif
 
 static void gl1_gfx_set_viewport_wrapper(void *data, unsigned viewport_width,
@@ -1489,7 +1531,8 @@ static unsigned gl1_get_alignment(unsigned pitch)
 static bool gl1_overlay_load(void *data,
       const void *image_data, unsigned num_images)
 {
-   unsigned i, j;
+   size_t i;
+   int j;
    gl1_t *gl = (gl1_t*)data;
    const struct texture_image *images =
       (const struct texture_image*)image_data;
@@ -1497,17 +1540,12 @@ static bool gl1_overlay_load(void *data,
    if (!gl)
       return false;
 
-   gl1_context_bind_hw_render(gl, false);
-
    gl1_free_overlay(gl);
    gl->overlay_tex = (GLuint*)
       calloc(num_images, sizeof(*gl->overlay_tex));
 
    if (!gl->overlay_tex)
-   {
-      gl1_context_bind_hw_render(gl, true);
       return false;
-   }
 
    gl->overlay_vertex_coord = (GLfloat*)
       calloc(2 * 4 * num_images, sizeof(GLfloat));
@@ -1543,20 +1581,22 @@ static bool gl1_overlay_load(void *data,
          gl->overlay_color_coord[16 * i + j] = 1.0f;
    }
 
-   gl1_context_bind_hw_render(gl, true);
    return true;
 }
 
 static void gl1_overlay_enable(void *data, bool state)
 {
-   gl1_t *gl           = (gl1_t*)data;
+   gl1_t *gl     = (gl1_t*)data;
 
    if (!gl)
       return;
 
-   gl->overlay_enable = state;
+   if (state)
+      gl->flags |=  GL1_FLAG_OVERLAY_ENABLE;
+   else
+      gl->flags &= ~GL1_FLAG_OVERLAY_ENABLE;
 
-   if (gl->fullscreen && gl->ctx_driver->show_mouse)
+   if ((gl->flags & GL1_FLAG_FULLSCREEN) && gl->ctx_driver->show_mouse)
       gl->ctx_driver->show_mouse(gl->ctx_data, state);
 }
 
@@ -1565,7 +1605,12 @@ static void gl1_overlay_full_screen(void *data, bool enable)
    gl1_t *gl = (gl1_t*)data;
 
    if (gl)
-      gl->overlay_full_screen = enable;
+   {
+      if (enable)
+         gl->flags |=  GL1_FLAG_OVERLAY_FULLSCREEN;
+      else
+         gl->flags &= ~GL1_FLAG_OVERLAY_FULLSCREEN;
+   }
 }
 
 static void gl1_overlay_set_alpha(void *data, unsigned image, float mod)
@@ -1595,7 +1640,6 @@ static const video_overlay_interface_t gl1_overlay_interface = {
 static void gl1_get_overlay_interface(void *data,
       const video_overlay_interface_t **iface)
 {
-   (void)data;
    *iface = &gl1_overlay_interface;
 }
 
@@ -1628,9 +1672,6 @@ video_driver_t video_gl1 = {
 
 #ifdef HAVE_OVERLAY
    gl1_get_overlay_interface,
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-  NULL,
 #endif
   gl1_gfx_get_poke_interface,
   gl1_wrap_type_to_enum,
