@@ -22,6 +22,7 @@
 #include <ogcsys.h>
 
 #include <libretro.h>
+#include <verbosity.h>
 #include <streams/interface_stream.h>
 
 #ifdef HAVE_CONFIG_H
@@ -327,9 +328,16 @@ static void gx_set_video_mode(void *data, unsigned fbWidth, unsigned lines,
    g_draw_done                 = false;
    /* wait for next even field */
    /* this prevents screen artifacts when switching
-    * between interlaced & non-interlaced modes */
-   do VIDEO_WaitVSync();
-   while (!VIDEO_GetNextField());
+    * between interlaced & non-interlaced modes.
+    *
+    * But move on if it takes over 3 frames as sometimes under dolphin
+    * this stays at constant value.
+    */
+   for (i = 0; i < 3; i++) {
+     VIDEO_WaitVSync();
+     if (VIDEO_GetNextField())
+       break;
+   }
 
    VIDEO_SetBlack(true);
    VIDEO_Flush();
@@ -600,7 +608,7 @@ static void gx_set_aspect_ratio(void *data, unsigned aspect_ratio_idx)
 }
 
 static void gx_get_video_output_size(void *data,
-      unsigned *width, unsigned *height)
+      unsigned *width, unsigned *height, char *desc, size_t desc_len)
 {
    global_t *global = global_get_ptr();
    if (!global)
@@ -620,7 +628,9 @@ static void gx_get_video_output_size(void *data,
 
 static void setup_video_mode(gx_video_t *gx)
 {
-   unsigned width, height;
+   unsigned width  = 0;
+   unsigned height = 0;
+   char desc[64]   = {0};
 
    if (!gx->framebuf[0])
    {
@@ -633,7 +643,7 @@ static void setup_video_mode(gx_video_t *gx)
    gx->orientation = ORIENTATION_NORMAL;
    OSInitThreadQueue(&g_video_cond);
 
-   gx_get_video_output_size(gx, &width, &height);
+   gx_get_video_output_size(gx, &width, &height, desc, sizeof(desc));
    gx_set_video_mode(gx, width, height, true);
 }
 
@@ -1372,7 +1382,11 @@ static const video_poke_interface_t gx_poke_interface = {
    NULL,                         /* grab_mouse_toggle */
    NULL,                         /* get_current_shader */
    NULL,                         /* get_current_software_framebuffer */
-   NULL                          /* get_hw_render_interface */
+   NULL,                         /* get_hw_render_interface */
+   NULL,                         /* set_hdr_max_nits */
+   NULL,                         /* set_hdr_paper_white_nits */
+   NULL,                         /* set_hdr_contrast */
+   NULL                          /* set_hdr_expand_gamut */
 };
 
 static void gx_get_poke_interface(void *data,
@@ -1796,9 +1810,6 @@ video_driver_t video_gx = {
    NULL, /* read_frame_raw */
 #ifdef HAVE_OVERLAY
    gx_get_overlay_interface,
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-  NULL,
 #endif
    gx_get_poke_interface,
 };
