@@ -171,7 +171,7 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
 
    if (frame)
    {
-      if(!(vita->texture&&vita2d_texture_get_datap(vita->texture)==frame))
+      if (!(vita->texture&&vita2d_texture_get_datap(vita->texture)==frame))
       {
          unsigned i;
          unsigned int stride;
@@ -252,7 +252,7 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
       menu_driver_frame(menu_is_alive, video_info);
 #endif
 
-      if(vita->menu.texture)
+      if (vita->menu.texture)
       {
          if (vita->fullscreen)
             vita2d_draw_texture_scale(vita->menu.texture,
@@ -297,7 +297,7 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
       gfx_widgets_frame(video_info);
 #endif
 
-   if(!string_is_empty(msg))
+   if (!string_is_empty(msg))
       font_driver_render_msg(vita, msg, NULL, NULL);
 
    vita2d_end_drawing();
@@ -370,7 +370,13 @@ static bool vita2d_gfx_set_shader(void *data,
 static void vita2d_set_projection(vita_video_t *vita,
       struct video_ortho *ortho, bool allow_rotate)
 {
-   math_matrix_4x4 rot;
+   static math_matrix_4x4 rot     = {
+      { 0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    0.0f ,
+        0.0f,     0.0f,    0.0f,    1.0f }
+   };
+   float radians, cosine, sine;
 
    /* Calculate projection. */
    matrix_4x4_ortho(vita->mvp_no_rot, ortho->left, ortho->right,
@@ -382,7 +388,13 @@ static void vita2d_set_projection(vita_video_t *vita,
       return;
    }
 
-   matrix_4x4_rotate_z(rot, M_PI * vita->rotation / 180.0f);
+   radians                 = M_PI * vita->rotation / 180.0f;
+   cosine                  = cosf(radians);
+   sine                    = sinf(radians);
+   MAT_ELEM_4X4(rot, 0, 0) = cosine;
+   MAT_ELEM_4X4(rot, 0, 1) = -sine;
+   MAT_ELEM_4X4(rot, 1, 0) = sine;
+   MAT_ELEM_4X4(rot, 1, 1) = cosine;
    matrix_4x4_multiply(vita->mvp, rot, vita->mvp_no_rot);
 }
 
@@ -770,7 +782,7 @@ static bool vita_get_current_sw_framebuffer(void *data,
    if (!vita->texture || vita->width != framebuffer->width ||
          vita->height != framebuffer->height)
    {
-      if(vita->texture)
+      if (vita->texture)
       {
          vita2d_wait_rendering_done();
          vita2d_free_texture(vita->texture);
@@ -824,7 +836,11 @@ static const video_poke_interface_t vita_poke_interface = {
    NULL,
    NULL,
    vita_get_current_sw_framebuffer,
-   NULL
+   NULL,
+   NULL, /* set_hdr_max_nits */
+   NULL, /* set_hdr_paper_white_nits */
+   NULL, /* set_hdr_contrast */
+   NULL  /* set_hdr_expand_gamut */
  };
 
 static void vita2d_gfx_get_poke_interface(void *data,
@@ -996,9 +1012,6 @@ video_driver_t video_vita2d = {
    NULL, /* read_frame_raw */
 #ifdef HAVE_OVERLAY
    vita2d_get_overlay_interface,
-#endif
-#ifdef HAVE_VIDEO_LAYOUT
-  NULL,
 #endif
    vita2d_gfx_get_poke_interface,
    NULL,
