@@ -19,7 +19,6 @@
  */
 
 #include <stdint.h>
-#include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <math.h>
@@ -37,6 +36,10 @@
 #include <lists/dir_list.h>
 #include <string/stdstring.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "../../configuration.h"
 #include "../../verbosity.h"
 #include "../../frontend/frontend_driver.h"
@@ -46,13 +49,7 @@
 #include "../common/egl_common.h"
 #endif
 
-#if defined(HAVE_OPENGL) || defined(HAVE_OPENGLES)
 #include "../common/gl_common.h"
-#endif
-
-#ifdef HAVE_CONFIG_H
-#include "../../config.h"
-#endif
 
 #ifdef HAVE_OPENGLES
 
@@ -71,17 +68,16 @@ typedef struct gfx_ctx_drm_data
 #ifdef HAVE_EGL
    egl_ctx_data_t egl;
 #endif
-   int fd;
-   int interval;
-   unsigned fb_width;
-   unsigned fb_height;
-
-   bool core_hw_context_enable;
-   bool waiting_for_flip;
    struct gbm_bo *bo;
    struct gbm_bo *next_bo;
    struct gbm_surface *gbm_surface;
    struct gbm_device  *gbm_dev;
+   int fd;
+   int interval;
+   unsigned fb_width;
+   unsigned fb_height;
+   bool core_hw_context_enable;
+   bool waiting_for_flip;
 } gfx_ctx_drm_data_t;
 
 struct drm_fb
@@ -96,66 +92,67 @@ struct drm_fb
  */
 typedef struct hdmi_timings
 {
-   int h_active_pixels; /* horizontal pixels (width) */
-   int h_sync_polarity; /* invert hsync polarity */
-   int h_front_porch;   /* horizontal forward padding from DE acitve edge */
-   int h_sync_pulse;    /* hsync pulse width in pixel clocks */
-   int h_back_porch;    /* vertical back padding from DE active edge */
-   int v_active_lines;  /* vertical pixels height (lines) */
-   int v_sync_polarity; /* invert vsync polarity */
-   int v_front_porch;   /* vertical forward padding from DE active edge */
-   int v_sync_pulse;    /* vsync pulse width in pixel clocks */
-   int v_back_porch;    /* vertical back padding from DE active edge */
-   int v_sync_offset_a; /* leave at zero */
-   int v_sync_offset_b; /* leave at zero */
-   int pixel_rep;       /* leave at zero */
-   int frame_rate;      /* screen refresh rate in Hz */
-   int interlaced;      /* leave at zero */
-   int pixel_freq;      /* clock frequency (width*height*framerate) */
+   int h_active_pixels; /* Horizontal pixels (width) */
+   int h_sync_polarity; /* Invert HSync polarity */
+   int h_front_porch;   /* Horizontal forward padding from DE active edge */
+   int h_sync_pulse;    /* HSync pulse width in pixel clocks */
+   int h_back_porch;    /* Vertical back padding from DE active edge */
+   int v_active_lines;  /* Vertical pixels height (lines) */
+   int v_sync_polarity; /* Invert vsync polarity */
+   int v_front_porch;   /* Vertical forward padding from DE active edge */
+   int v_sync_pulse;    /* VSync pulse width in pixel clocks */
+   int v_back_porch;    /* Vertical back padding from DE active edge */
+   int v_sync_offset_a; /* Leave at zero */
+   int v_sync_offset_b; /* Leave at zero */
+   int pixel_rep;       /* Leave at zero */
+   int frame_rate;      /* Screen refresh rate in Hz */
+   int interlaced;      /* Leave at zero */
+   int pixel_freq;      /* Clock frequency (width*height*framerate) */
    int aspect_ratio;
 } hdmi_timings_t;
 
 static enum gfx_ctx_api drm_api           = GFX_CTX_NONE;
 static drmModeModeInfo gfx_ctx_crt_switch_mode;
 
-/* Load custom hdmi timings from config */
-bool gfx_ctx_drm_load_mode(drmModeModeInfoPtr modeInfo)
+/* Load custom HDMI timings from config */
+static bool gfx_ctx_drm_load_mode(drmModeModeInfoPtr modeInfo)
 {
-   int ret;
-   hdmi_timings_t timings;
-   settings_t *settings = config_get_ptr();
+   settings_t *settings     = config_get_ptr();
    char *crt_switch_timings = settings->arrays.crt_switch_timings;
 
-   if(modeInfo != NULL && !string_is_empty(crt_switch_timings)) {
-      ret = sscanf(crt_switch_timings, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
+   if (modeInfo && !string_is_empty(crt_switch_timings))
+   {
+	   hdmi_timings_t timings;
+      int ret               = sscanf(crt_switch_timings, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
                    &timings.h_active_pixels, &timings.h_sync_polarity, &timings.h_front_porch,
                    &timings.h_sync_pulse, &timings.h_back_porch,
                    &timings.v_active_lines, &timings.v_sync_polarity, &timings.v_front_porch,
                    &timings.v_sync_pulse, &timings.v_back_porch,
                    &timings.v_sync_offset_a, &timings.v_sync_offset_b, &timings.pixel_rep, &timings.frame_rate,
                    &timings.interlaced, &timings.pixel_freq, &timings.aspect_ratio);
-      if (ret != 17) {
+      if (ret != 17)
+      {
          RARCH_ERR("[DRM]: malformed mode requested: %s\n", crt_switch_timings);
          return false;
       }
 
       memset(modeInfo, 0, sizeof(drmModeModeInfo));
-      modeInfo->clock = timings.pixel_freq / 1000;
-      modeInfo->hdisplay = timings.h_active_pixels;
+      modeInfo->clock       = timings.pixel_freq / 1000;
+      modeInfo->hdisplay    = timings.h_active_pixels;
       modeInfo->hsync_start = modeInfo->hdisplay + timings.h_front_porch;
-      modeInfo->hsync_end = modeInfo->hsync_start + timings.h_sync_pulse;
-      modeInfo->htotal = modeInfo->hsync_end + timings.h_back_porch;
-      modeInfo->hskew = 0;
-      modeInfo->vdisplay = timings.v_active_lines;
+      modeInfo->hsync_end   = modeInfo->hsync_start + timings.h_sync_pulse;
+      modeInfo->htotal      = modeInfo->hsync_end + timings.h_back_porch;
+      modeInfo->hskew       = 0;
+      modeInfo->vdisplay    = timings.v_active_lines;
       modeInfo->vsync_start = modeInfo->vdisplay + (timings.v_front_porch * (timings.interlaced ? 2 : 1));
-      modeInfo->vsync_end = modeInfo->vsync_start + (timings.v_sync_pulse * (timings.interlaced ? 2 : 1));
-      modeInfo->vtotal = modeInfo->vsync_end + (timings.v_back_porch * (timings.interlaced ? 2 : 1));
-      modeInfo->vscan = 0; /* TODO: ?? */
-      modeInfo->vrefresh = timings.frame_rate;
-      modeInfo->flags = timings.interlaced ? DRM_MODE_FLAG_INTERLACE : 0;
-      modeInfo->flags |= timings.v_sync_polarity ? DRM_MODE_FLAG_NVSYNC : DRM_MODE_FLAG_PVSYNC;
-      modeInfo->flags |= timings.h_sync_polarity ? DRM_MODE_FLAG_NHSYNC : DRM_MODE_FLAG_PHSYNC;
-      modeInfo->type = 0;
+      modeInfo->vsync_end   = modeInfo->vsync_start + (timings.v_sync_pulse * (timings.interlaced ? 2 : 1));
+      modeInfo->vtotal      = modeInfo->vsync_end + (timings.v_back_porch * (timings.interlaced ? 2 : 1));
+      modeInfo->vscan       = 0; /* TODO: ?? */
+      modeInfo->vrefresh    = timings.frame_rate;
+      modeInfo->flags       = timings.interlaced ? DRM_MODE_FLAG_INTERLACE : 0;
+      modeInfo->flags      |= timings.v_sync_polarity ? DRM_MODE_FLAG_NVSYNC : DRM_MODE_FLAG_PVSYNC;
+      modeInfo->flags      |= timings.h_sync_polarity ? DRM_MODE_FLAG_NHSYNC : DRM_MODE_FLAG_PHSYNC;
+      modeInfo->type        = 0;
       snprintf(modeInfo->name, DRM_DISPLAY_MODE_LEN, "CRT_%ux%u_%u",
                modeInfo->hdisplay, modeInfo->vdisplay, modeInfo->vrefresh);
 
@@ -200,7 +197,7 @@ static struct drm_fb *drm_fb_get_from_bo(struct gbm_bo *bo)
    return fb;
 
 error:
-   RARCH_ERR("[KMS]: Failed to create FB: %s\n", strerror(errno));
+   RARCH_ERR("[KMS]: Failed to create FB.\n");
    free(fb);
    return NULL;
 }
@@ -439,7 +436,7 @@ nextgpu:
      - custom timings from configuration
      - else the current video mode from the CRTC
      - otherwise pick first connector mode */
-   if(gfx_ctx_drm_load_mode(&gfx_ctx_crt_switch_mode))
+   if (gfx_ctx_drm_load_mode(&gfx_ctx_crt_switch_mode))
    {
       drm->fb_width  = gfx_ctx_crt_switch_mode.hdisplay;
       drm->fb_height = gfx_ctx_crt_switch_mode.vdisplay;
@@ -671,7 +668,7 @@ static bool gfx_ctx_drm_egl_set_video_mode(gfx_ctx_drm_data_t *drm)
       case GFX_CTX_OPENGL_API:
       case GFX_CTX_OPENGL_ES_API:
 #if defined(HAVE_OPENGL) || defined(HAVE_OPENGLES)
-         glClear(GL_COLOR_BUFFER_BIT);
+         gl_clear();
 #endif
          break;
       case GFX_CTX_NONE:
@@ -695,12 +692,12 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
       bool fullscreen)
 {
    float refresh_mod;
-   int i, ret                  = 0;
-   struct drm_fb *fb           = NULL;
-   gfx_ctx_drm_data_t *drm     = (gfx_ctx_drm_data_t*)data;
-   settings_t *settings        = config_get_ptr();
+   int i, ret                      = 0;
+   struct drm_fb *fb               = NULL;
+   gfx_ctx_drm_data_t *drm         = (gfx_ctx_drm_data_t*)data;
+   settings_t *settings            = config_get_ptr();
    unsigned black_frame_insertion  = settings->uints.video_black_frame_insertion;
-   float video_refresh_rate    = settings->floats.video_refresh_rate;
+   float video_refresh_rate        = settings->floats.video_refresh_rate;
 
    if (!drm)
       return false;
@@ -710,20 +707,20 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
    /* If we use black frame insertion,
     * we fake a 60 Hz monitor for 120 Hz one,
     * etc, so try to match that. */
-   refresh_mod = 1.0f / (black_frame_insertion + 1.0f);
+   refresh_mod                     = 1.0f / (black_frame_insertion + 1.0f);
 
    /* Find desired video mode, and use that.
     * If not fullscreen, we get desired windowed size,
     * which is not appropriate. */
    if ((width == 0 && height == 0) || !fullscreen)
-      g_drm_mode = &g_drm_connector->modes[0];
+      g_drm_mode                   = &g_drm_connector->modes[0];
    else
    {
-      /* check if custom hdmi timings were asked */
-      if(gfx_ctx_crt_switch_mode.vdisplay > 0)
+      /* check if custom HDMI timings were asked */
+      if (gfx_ctx_crt_switch_mode.vdisplay > 0)
       {
+         g_drm_mode                = &gfx_ctx_crt_switch_mode;
          RARCH_LOG("[DRM]: custom mode requested: %s\n", gfx_ctx_crt_switch_mode.name);
-         g_drm_mode = &gfx_ctx_crt_switch_mode;
       }
       else
       {
@@ -732,26 +729,27 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
           * Lower resolutions tend to have multiple supported
           * refresh rates as well.
           */
-         float minimum_fps_diff = 0.0f;
-         float mode_vrefresh    = 0.0f;
-         drmModeModeInfo *mode;
+         drmModeModeInfo *mode     = NULL;
+         float minimum_fps_diff    = 0.0f;
+         float mode_vrefresh       = 0.0f;
 
          /* Find best match. */
-         for (i = 0; i < g_drm_connector->count_modes; i++) {
+         for (i = 0; i < g_drm_connector->count_modes; i++)
+         {
             float diff;
-            mode = &g_drm_connector->modes[i];
+            mode                   = &g_drm_connector->modes[i];
 
-            if (width != mode->hdisplay ||
-               height != mode->vdisplay)
+            if (   (width  != mode->hdisplay)
+                || (height != mode->vdisplay))
                continue;
 
-            mode_vrefresh = drm_calc_refresh_rate(mode);
+            mode_vrefresh          = drm_calc_refresh_rate(mode);
+            diff                   = fabsf(refresh_mod * mode_vrefresh - video_refresh_rate);
 
-            diff = fabsf(refresh_mod * mode_vrefresh - video_refresh_rate);
-
-            if (!g_drm_mode || diff < minimum_fps_diff) {
-               g_drm_mode = mode;
-               minimum_fps_diff = diff;
+            if (!g_drm_mode || diff < minimum_fps_diff)
+            {
+               g_drm_mode          = mode;
+               minimum_fps_diff    = diff;
             }
          }
       }
@@ -764,11 +762,11 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
       goto error;
    }
 
-   drm->fb_width    = g_drm_mode->hdisplay;
-   drm->fb_height   = g_drm_mode->vdisplay;
+   drm->fb_width                   = g_drm_mode->hdisplay;
+   drm->fb_height                  = g_drm_mode->vdisplay;
 
    /* Create GBM surface. */
-   drm->gbm_surface = gbm_surface_create(
+   drm->gbm_surface                = gbm_surface_create(
          drm->gbm_dev,
          drm->fb_width,
          drm->fb_height,
@@ -788,14 +786,11 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
 
    drm->bo   = gbm_surface_lock_front_buffer(drm->gbm_surface);
 
-   fb        = (struct drm_fb*)gbm_bo_get_user_data(drm->bo);
+   if (!(fb = (struct drm_fb*)gbm_bo_get_user_data(drm->bo)))
+      fb     = drm_fb_get_from_bo(drm->bo);
 
-   if (!fb)
-      fb   = drm_fb_get_from_bo(drm->bo);
-
-   ret     = drmModeSetCrtc(g_drm_fd,
-         g_crtc_id, fb->fb_id, 0, 0, &g_connector_id, 1, g_drm_mode);
-   if (ret < 0)
+   if ((ret  = drmModeSetCrtc(g_drm_fd,
+         g_crtc_id, fb->fb_id, 0, 0, &g_connector_id, 1, g_drm_mode)) < 0)
       goto error;
 
    return true;
@@ -914,9 +909,8 @@ static bool gfx_ctx_drm_bind_api(void *video_driver,
 
 static void gfx_ctx_drm_bind_hw_render(void *data, bool enable)
 {
-   gfx_ctx_drm_data_t *drm     = (gfx_ctx_drm_data_t*)data;
-
 #ifdef HAVE_EGL
+   gfx_ctx_drm_data_t *drm     = (gfx_ctx_drm_data_t*)data;
    egl_bind_hw_render(&drm->egl, enable);
 #endif
 }
