@@ -41,6 +41,20 @@ float cocoa_screen_get_backing_scale_factor(void);
 #endif
 
 static bool apple_key_state[MAX_KEYS];
+
+/* Send keyboard inputs directly using RETROK_* codes
+ * Used by the iOS custom keyboard implementation */
+void apple_direct_input_keyboard_event(bool down,
+      unsigned code, uint32_t character, uint32_t mod, unsigned device)
+{
+    int apple_key              = rarch_keysym_lut[code];
+    apple_key_state[apple_key] = down;
+    input_keyboard_event(down,
+          code,
+          character, (enum retro_mod)mod, device);
+}
+
+
 #if TARGET_OS_IPHONE
 /* TODO/FIXME - static globals */
 static bool small_keyboard_active = false;
@@ -312,7 +326,7 @@ static const unsigned char MAC_NATIVE_TO_HID[128] = {
 void apple_input_keyboard_event(bool down,
       unsigned code, uint32_t character, uint32_t mod, unsigned device)
 {
-   code                         = HIDKEY(code);
+   code                  = HIDKEY(code);
    if (code == 0 || code >= MAX_KEYS)
       return;
 
@@ -377,7 +391,7 @@ static int16_t cocoa_input_state(
       const input_device_driver_t *joypad,
       const input_device_driver_t *sec_joypad,
       rarch_joypad_info_t *joypad_info,
-      const struct retro_keybind **binds,
+      const retro_keybind_set *binds,
       bool keyboard_mapping_blocked,
       unsigned port,
       unsigned device,
@@ -410,12 +424,43 @@ static int16_t cocoa_input_state(
          if (binds[port][id].valid)
          {
             if (id < RARCH_BIND_LIST_END)
-               if (apple_key_state[rarch_keysym_lut[binds[port][id].key]])
-                  return 1;
+               if (!keyboard_mapping_blocked || (id == RARCH_GAME_FOCUS_TOGGLE))
+                  if (apple_key_state[rarch_keysym_lut[binds[port][id].key]])
+                     return 1;
+
          }
          break;
       case RETRO_DEVICE_ANALOG:
+         {
+            int16_t ret           = 0;
+            int id_minus_key      = 0;
+            int id_plus_key       = 0;
+            unsigned id_minus     = 0;
+            unsigned id_plus      = 0;
+            bool id_plus_valid    = false;
+            bool id_minus_valid   = false;
+
+            input_conv_analog_id_to_bind_id(idx, id, id_minus, id_plus);
+
+            id_minus_valid        = binds[port][id_minus].valid;
+            id_plus_valid         = binds[port][id_plus].valid;
+            id_minus_key          = binds[port][id_minus].key;
+            id_plus_key           = binds[port][id_plus].key;
+
+            if (id_plus_valid && id_plus_key < RETROK_LAST)
+            {
+               if (apple_key_state[rarch_keysym_lut[(enum retro_key)id_plus_key]])
+                  ret = 0x7fff;
+            }
+            if (id_minus_valid && id_minus_key < RETROK_LAST)
+            {
+               if (apple_key_state[rarch_keysym_lut[(enum retro_key)id_minus_key]])
+                  ret += -0x7fff;
+            }
+            return ret;
+         }
          break;
+
       case RETRO_DEVICE_KEYBOARD:
          return (id < RETROK_LAST) && apple_key_state[rarch_keysym_lut[(enum retro_key)id]];
       case RETRO_DEVICE_MOUSE:
@@ -433,8 +478,22 @@ static int16_t cocoa_input_state(
                      return apple->window_pos_x * cocoa_screen_get_backing_scale_factor();
 #endif
                   }
+#ifdef IOS
+#ifdef HAVE_IOS_TOUCHMOUSE
+                  if (apple->window_pos_x > 0)
+                  {
+                     val = apple->window_pos_x - apple->mouse_x_last;
+                     apple->mouse_x_last = apple->window_pos_x;
+                  }
+                  else
+                     val = apple->mouse_rel_x;
+#else
+                  val = apple->mouse_rel_x;
+#endif
+#else
                   val = apple->window_pos_x - apple->mouse_x_last;
                   apple->mouse_x_last = apple->window_pos_x;
+#endif
                   return val;
                case RETRO_DEVICE_ID_MOUSE_Y:
                   if (device == RARCH_DEVICE_MOUSE_SCREEN)
@@ -445,8 +504,22 @@ static int16_t cocoa_input_state(
                      return apple->window_pos_y * cocoa_screen_get_backing_scale_factor();
 #endif
                   }
+#ifdef IOS
+#ifdef HAVE_IOS_TOUCHMOUSE
+                  if (apple->window_pos_y > 0)
+                  {
+                     val = apple->window_pos_y - apple->mouse_y_last;
+                     apple->mouse_y_last = apple->window_pos_y;
+                  }
+                  else
+                     val = apple->mouse_rel_y;
+#else
+                  val    = apple->mouse_rel_y;
+#endif
+#else
                   val = apple->window_pos_y - apple->mouse_y_last;
                   apple->mouse_y_last = apple->window_pos_y;
+#endif
                   return val;
                case RETRO_DEVICE_ID_MOUSE_LEFT:
                   return apple->mouse_buttons & 1;
@@ -466,36 +539,36 @@ static int16_t cocoa_input_state(
       case RETRO_DEVICE_POINTER:
       case RARCH_DEVICE_POINTER_SCREEN:
          {
-            const bool want_full = (device == RARCH_DEVICE_POINTER_SCREEN);
 
             if (idx < apple->touch_count && (idx < MAX_TOUCHES))
             {
-               int16_t x, y;
                const cocoa_touch_data_t *touch = (const cocoa_touch_data_t *)
                   &apple->touches[idx];
 
-               if (!touch)
-                  return 0;
-
-               x = touch->fixed_x;
-               y = touch->fixed_y;
-
-               if (want_full)
+               if (touch)
                {
-                  x = touch->full_x;
-                  y = touch->full_y;
-               }
+                  int16_t x, y;
+                  const bool want_full         = (device == RARCH_DEVICE_POINTER_SCREEN);
 
-               switch (id)
-               {
-                  case RETRO_DEVICE_ID_POINTER_PRESSED:
-                     return (x != -0x8000) && (y != -0x8000);
-                  case RETRO_DEVICE_ID_POINTER_X:
-                     return x;
-                  case RETRO_DEVICE_ID_POINTER_Y:
-                     return y;
-                  case RETRO_DEVICE_ID_POINTER_COUNT:
-                     return apple->touch_count;
+                  switch (id)
+                  {
+                     case RETRO_DEVICE_ID_POINTER_PRESSED:
+                        x = touch->fixed_x;
+                        y = touch->fixed_y;
+
+                        if (want_full)
+                        {
+                           x = touch->full_x;
+                           y = touch->full_y;
+                        }
+                        return (x != -0x8000) && (y != -0x8000); /* Inside? */
+                     case RETRO_DEVICE_ID_POINTER_X:
+                        return want_full ? touch->full_x : touch->fixed_x;
+                     case RETRO_DEVICE_ID_POINTER_Y:
+                        return want_full ? touch->full_y : touch->fixed_y;
+                     case RETRO_DEVICE_ID_POINTER_COUNT:
+                        return apple->touch_count;
+                  }
                }
             }
          }
@@ -522,11 +595,11 @@ static void cocoa_input_free(void *data)
 static uint64_t cocoa_input_get_capabilities(void *data)
 {
    return
-      (1 << RETRO_DEVICE_JOYPAD)   |
-      (1 << RETRO_DEVICE_MOUSE)    |
-      (1 << RETRO_DEVICE_KEYBOARD) |
-      (1 << RETRO_DEVICE_POINTER)  |
-      (1 << RETRO_DEVICE_ANALOG);
+        (1 << RETRO_DEVICE_JOYPAD)
+      | (1 << RETRO_DEVICE_MOUSE)
+      | (1 << RETRO_DEVICE_KEYBOARD)
+      | (1 << RETRO_DEVICE_POINTER)
+      | (1 << RETRO_DEVICE_ANALOG);
 }
 
 input_driver_t input_cocoa = {
@@ -539,5 +612,6 @@ input_driver_t input_cocoa = {
    cocoa_input_get_capabilities,
    "cocoa",
    NULL,                         /* grab_mouse */
+   NULL,
    NULL
 };
