@@ -22,83 +22,179 @@
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "input_types.h"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include <boolean.h>
 #include <retro_common_api.h>
 #include <retro_inline.h>
 #include <libretro.h>
 #include <retro_miscellaneous.h>
+#include <streams/interface_stream.h>
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif /* HAVE_CONFIG_H */
 
 #include "input_defines.h"
+#include "input_types.h"
+#ifdef HAVE_OVERLAY
+#include "input_overlay.h"
+#endif
+#include "input_osk.h"
 
 #include "../msg_hash.h"
 #include "include/hid_types.h"
 #include "include/hid_driver.h"
 #include "include/gamepad.h"
+#include "../configuration.h"
+#include "../performance_counters.h"
+
+#ifdef HAVE_COMMAND
+#include "../command.h"
+#endif
+
+#if defined(ANDROID)
+#define DEFAULT_MAX_PADS 8
+#define ANDROID_KEYBOARD_PORT DEFAULT_MAX_PADS
+#elif defined(_3DS)
+#define DEFAULT_MAX_PADS 1
+#elif defined(SWITCH) || defined(HAVE_LIBNX)
+#define DEFAULT_MAX_PADS 8
+#elif defined(WIIU)
+#ifdef WIIU_HID
+#define DEFAULT_MAX_PADS 16
+#else
+#define DEFAULT_MAX_PADS 5
+#endif /* WIIU_HID */
+#elif defined(DJGPP)
+#define DEFAULT_MAX_PADS 1
+#define DOS_KEYBOARD_PORT DEFAULT_MAX_PADS
+#elif defined(XENON)
+#define DEFAULT_MAX_PADS 4
+#elif defined(VITA) || defined(SN_TARGET_PSP2) || defined(ORBIS)
+#define DEFAULT_MAX_PADS 4
+#elif defined(PSP)
+#define DEFAULT_MAX_PADS 1
+#elif defined(PS2)
+#define DEFAULT_MAX_PADS 8
+#elif defined(GEKKO) || defined(HW_RVL)
+#define DEFAULT_MAX_PADS 4
+#elif defined(HAVE_ODROIDGO2)
+#define DEFAULT_MAX_PADS 8
+#elif defined(__linux__) || (defined(BSD) && !defined(__MACH__))
+#define DEFAULT_MAX_PADS 8
+#elif defined(__QNX__)
+#define DEFAULT_MAX_PADS 8
+#elif defined(__PS3__)
+#define DEFAULT_MAX_PADS 7
+#elif defined(_XBOX)
+#define DEFAULT_MAX_PADS 4
+#elif defined(HAVE_XINPUT) && !defined(HAVE_DINPUT)
+#define DEFAULT_MAX_PADS 4
+#elif defined(DINGUX)
+#define DEFAULT_MAX_PADS 2
+#else
+#define DEFAULT_MAX_PADS 16
+#endif /* defined(ANDROID) */
+
+#define MAPPER_GET_KEY(state, key) (((state)->keys[(key) / 32] >> ((key) % 32)) & 1)
+#define MAPPER_SET_KEY(state, key) (state)->keys[(key) / 32] |= 1 << ((key) % 32)
+#define MAPPER_UNSET_KEY(state, key) (state)->keys[(key) / 32] &= ~(1 << ((key) % 32))
+
+#define INHERIT_JOYAXIS(binds) (((binds)[x_plus].joyaxis == (binds)[x_minus].joyaxis) || (  (binds)[y_plus].joyaxis == (binds)[y_minus].joyaxis))
 
 RETRO_BEGIN_DECLS
 
-struct retro_keybind
+enum rarch_movie_type
 {
-   /**
-    * Human-readable label for the control.
-    */
-   char     *joykey_label;
-
-   /**
-    * Human-readable label for an analog axis.
-    */
-   char     *joyaxis_label;
-
-   /**
-    * Joypad axis. Negative and positive axes are both represented by this variable.
-    */
-   uint32_t joyaxis;
-
-   /**
-    * Default joy axis binding value for resetting bind to default.
-    */
-   uint32_t def_joyaxis;
-
-   /**
-    * Used by input_{push,pop}_analog_dpad().
-    */
-   uint32_t orig_joyaxis;
-
-   enum msg_hash_enums enum_idx;
-
-   enum retro_key key;
-
-   uint16_t id;
-
-   /**
-    * What mouse button ID has been mapped to this control.
-    */
-   uint16_t mbutton;
-
-   /**
-    * Joypad key. Joypad POV (hats) are embedded into this key as well.
-    **/
-   uint16_t joykey;
-
-   /**
-    * Default key binding value (for resetting bind).
-    */
-   uint16_t def_joykey;
-
-   /**
-    * Determines whether or not the binding is valid.
-    */
-   bool valid;
+   RARCH_MOVIE_PLAYBACK = 0,
+   RARCH_MOVIE_RECORD
 };
 
-extern struct retro_keybind input_config_binds[MAX_USERS][RARCH_BIND_LIST_END];
-extern struct retro_keybind input_autoconf_binds[MAX_USERS][RARCH_BIND_LIST_END];
+#ifdef HAVE_BSV_MOVIE
+enum bsv_flags
+{
+   BSV_FLAG_MOVIE_START_RECORDING    = (1 << 0),
+   BSV_FLAG_MOVIE_START_PLAYBACK     = (1 << 1),
+   BSV_FLAG_MOVIE_PLAYBACK           = (1 << 2),
+   BSV_FLAG_MOVIE_RECORDING          = (1 << 3),
+   BSV_FLAG_MOVIE_END                = (1 << 4),
+   BSV_FLAG_MOVIE_EOF_EXIT           = (1 << 5)
+};
+
+struct bsv_state
+{
+   uint8_t flags;
+   /* Movie playback/recording support. */
+   char movie_auto_path[PATH_MAX_LENGTH];
+   /* Immediate playback/recording. */
+   char movie_start_path[PATH_MAX_LENGTH];
+};
+
+/* These data are always little-endian. */
+struct bsv_key_data {
+  uint8_t down;
+  uint16_t mod;
+  uint8_t _padding;
+  uint32_t code;
+  uint32_t character;
+};
+
+typedef struct bsv_key_data bsv_key_data_t;
+
+struct bsv_movie
+{
+   intfstream_t *file;
+   uint8_t *state;
+   /* A ring buffer keeping track of positions
+    * in the file for each frame. */
+   size_t *frame_pos;
+   size_t frame_mask;
+   size_t frame_ptr;
+   size_t min_file_pos;
+   size_t state_size;
+   int64_t identifier;
+
+   /* Staging variables for keyboard events */
+   uint8_t key_event_count;
+   bsv_key_data_t key_events[255];
+
+   /* Rewind state */
+   bool playback;
+   bool first_rewind;
+   bool did_rewind;
+};
+
+typedef struct bsv_movie bsv_movie_t;
+
+#define REPLAY_TOKEN_INVALID          '\0'
+#define REPLAY_TOKEN_REGULAR_FRAME    'f'
+#define REPLAY_TOKEN_CHECKPOINT_FRAME 'c'
+#endif
+
+/**
+ * line_complete callback (when carriage return is pressed)
+ *
+ * @param userdata User data which will be passed to subsequent callbacks.
+ * @param line      the line of input, which can be NULL.
+ **/
+typedef void (*input_keyboard_line_complete_t)(void *userdata,
+      const char *line);
+
+struct input_keyboard_line
+{
+   char *buffer;
+   void *userdata;
+   /** Line complete callback.
+    * Calls back after return is
+    * pressed with the completed line.
+    * Line can be NULL.
+    **/
+   input_keyboard_line_complete_t cb;
+   size_t ptr;
+   size_t size;
+   bool enabled;
+};
+
+extern retro_keybind_set input_config_binds[MAX_USERS];
+extern retro_keybind_set input_autoconf_binds[MAX_USERS];
 
 struct rarch_joypad_info
 {
@@ -116,10 +212,53 @@ typedef struct
    char name[256];
    char display_name[256];
    char config_path[PATH_MAX_LENGTH]; /* Path to the RetroArch config file */
-   char config_name[PATH_MAX_LENGTH]; /* Base name of the RetroArch config file */
+   char config_name[256]; /* Base name of the RetroArch config file */
    bool autoconfigured;
 } input_device_info_t;
 
+struct remote_message
+{
+   int port;
+   int device;
+   int index;
+   int id;
+   uint16_t state;
+};
+
+struct input_remote
+{
+#if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
+   int net_fd[MAX_USERS];
+#endif
+   bool state[RARCH_BIND_LIST_END];
+};
+
+
+typedef struct
+{
+   char display_name[256];
+} input_mouse_info_t;
+
+typedef struct input_remote input_remote_t;
+
+typedef struct input_remote_state
+{
+   /* This is a bitmask of (1 << key_bind_id). */
+   uint64_t buttons[MAX_USERS];
+   /* Left X, Left Y, Right X, Right Y */
+   int16_t analog[4][MAX_USERS];
+} input_remote_state_t;
+
+typedef struct input_list_element_t
+{
+   int16_t *state;
+   unsigned port;
+   unsigned device;
+   unsigned index;
+   unsigned int state_size;
+} input_list_element;
+
+
 /**
  * Organizes the functions and data structures of each driver that are accessed
  * by other parts of the input code. The input_driver structs are the "interface"
@@ -181,7 +320,7 @@ struct input_driver
          const input_device_driver_t *joypad_data,
          const input_device_driver_t *sec_joypad_data,
          rarch_joypad_info_t *joypad_info,
-         const struct retro_keybind **retro_keybinds,
+         const retro_keybind_set *retro_keybinds,
          bool keyboard_mapping_blocked,
          unsigned port, unsigned device, unsigned index, unsigned id);
 
@@ -197,10 +336,14 @@ struct input_driver
     * the sensor entirely, etc. This function pointer may be set to NULL if
     * setting sensor values is not supported.
     * 
-    * @param data    The input state struct
+    * @param data    
+    * The input state struct
     * @param port
-    * @param effect  Sensor action
-    * @param rate    Sensor rate update
+    * The port of the device
+    * @param effect  
+    * Sensor action
+    * @param rate    
+    * Sensor rate update
     * 
     * @return true if the operation is successful.
    **/
@@ -212,9 +355,12 @@ struct input_driver
     * function pointer may be set to NULL if retreiving sensor state is not
     * supported.
     * 
-    * @param data  The input state struct
+    * @param data  
+    * The input state struct
     * @param port
-    * @param id    Sensor ID
+    * The port of the device
+    * @param id    
+    * Sensor ID
     * 
     * @return The current state associated with the port and ID as a float
     **/
@@ -255,6 +401,12 @@ struct input_driver
     * @return True if the input driver has claimed stdin.
     */
    bool (*grab_stdin)(void *data);
+
+   /**
+    * Haptic feedback for touchscreen key presses. This function pointer can be
+    * set to NULL if haptic feedback / vibration is not supported.
+    */
+   void (*keypress_vibrate)(void);
 };
 
 struct rarch_joypad_driver
@@ -262,18 +414,123 @@ struct rarch_joypad_driver
    void *(*init)(void *data);
    bool (*query_pad)(unsigned);
    void (*destroy)(void);
-   int16_t (*button)(unsigned, uint16_t);
+   int32_t (*button)(unsigned, uint16_t);
    int16_t (*state)(rarch_joypad_info_t *joypad_info,
          const struct retro_keybind *binds, unsigned port);
    void (*get_buttons)(unsigned, input_bits_t *);
    int16_t (*axis)(unsigned, uint32_t);
    void (*poll)(void);
    bool (*set_rumble)(unsigned, enum retro_rumble_effect, uint16_t);
+   bool (*set_rumble_gain)(unsigned, unsigned);
    const char *(*name)(unsigned);
 
    const char *ident;
 };
 
+/**
+ * Callback for keypress events
+ * 
+ * @param userdata The user data that was passed through from the keyboard press callback.
+ * @param code      keycode
+ **/
+typedef bool (*input_keyboard_press_t)(void *userdata, unsigned code);
+
+struct input_keyboard_ctx_wait
+{
+   void *userdata;
+   input_keyboard_press_t cb;
+};
+
+enum input_driver_state_flags
+{
+   INP_FLAG_NONBLOCKING              = (1 << 0),
+   INP_FLAG_KB_LINEFEED_ENABLE       = (1 << 1),
+   INP_FLAG_KB_MAPPING_BLOCKED       = (1 << 2),
+   INP_FLAG_BLOCK_HOTKEY             = (1 << 3),
+   INP_FLAG_BLOCK_LIBRETRO_INPUT     = (1 << 4),
+   INP_FLAG_BLOCK_POINTER_INPUT      = (1 << 5),
+   INP_FLAG_GRAB_MOUSE_STATE         = (1 << 6),
+   INP_FLAG_OLD_ANALOG_DPAD_MODE_SET = (1 << 7),
+   INP_FLAG_OLD_LIBRETRO_DEVICE_SET  = (1 << 8),
+   INP_FLAG_REMAPPING_CACHE_ACTIVE   = (1 << 9),
+   INP_FLAG_DEFERRED_WAIT_KEYS       = (1 << 10)
+};
+
+typedef struct
+{
+   /**
+    * Array of timers, one for each entry in enum input_combo_type.
+    */
+   rarch_timer_t combo_timers[INPUT_COMBO_LAST];
+
+#if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
+   input_remote_state_t remote_st_ptr;        /* uint64_t alignment */
+#endif
+
+   /* pointers */
+#ifdef HAVE_HID
+   const void *hid_data;
+#endif
+   void *keyboard_press_data;
+   input_keyboard_line_t keyboard_line;                  /* ptr alignment */
+   input_keyboard_press_t keyboard_press_cb;             /* ptr alignment */
+   input_driver_t                *current_driver;
+   void                          *current_data;
+   const input_device_driver_t   *primary_joypad;        /* ptr alignment */
+   const input_device_driver_t   *secondary_joypad;      /* ptr alignment */
+   const retro_keybind_set *libretro_input_binds[MAX_USERS];
+#ifdef HAVE_COMMAND
+   command_t *command[MAX_CMD_DRIVERS];
+#endif
+#ifdef HAVE_BSV_MOVIE
+   bsv_movie_t     *bsv_movie_state_handle;              /* ptr alignment */
+#endif
+#ifdef HAVE_OVERLAY
+   input_overlay_t *overlay_ptr;
+   enum overlay_visibility *overlay_visibility;
+   float overlay_eightway_dpad_slopes[2];
+   float overlay_eightway_abxy_slopes[2];
+#endif
+   uint16_t flags;
+#ifdef HAVE_NETWORKGAMEPAD
+   input_remote_t *remote;
+#endif
+   char    *osk_grid[45];                                /* ptr alignment */ 
+#if defined(HAVE_TRANSLATE)
+#if defined(HAVE_ACCESSIBILITY)
+   int ai_gamepad_state[MAX_USERS];
+#endif
+#endif
+   int osk_ptr;
+   turbo_buttons_t turbo_btns; /* int32_t alignment */
+
+   input_mapper_t mapper;          /* uint32_t alignment */
+   input_device_info_t input_device_info[MAX_INPUT_DEVICES]; /* unsigned alignment */
+   input_mouse_info_t input_mouse_info[MAX_INPUT_DEVICES];
+   unsigned old_analog_dpad_mode[MAX_USERS];
+   unsigned old_libretro_device[MAX_USERS];
+   unsigned osk_last_codepoint;
+   unsigned osk_last_codepoint_len;
+   unsigned input_hotkey_block_counter;
+#ifdef HAVE_ACCESSIBILITY
+   unsigned gamepad_input_override;
+#endif
+
+   enum osk_type osk_idx;
+
+#ifdef HAVE_BSV_MOVIE
+   struct bsv_state bsv_movie_state;            /* char alignment */
+#endif
+
+   /* primitives */
+   bool analog_requested[MAX_USERS];
+   retro_bits_512_t keyboard_mapping_bits;    /* bool alignment */
+   input_game_focus_state_t game_focus_state; /* bool alignment */
+} input_driver_state_t;
+
+
+void input_driver_init_joypads(void);
+
 /**
  * Get an enumerated list of all input driver names
  *
@@ -282,73 +539,79 @@ struct rarch_joypad_driver
 const char* config_get_input_driver_options(void);
 
 /**
- * Sets the rumble state. Used by RETRO_ENVIRONMENT_GET_RUMBLE_INTERFACE.
+ * Sets the rumble state.
  * 
- * @param port      User number.
- * @param effect    Rumble effect.
- * @param strength  Strength of rumble effect.
+ * @param port          
+ * User number.
+ * @param joy_idx
+ * TODO/FIXME ???
+ * @param effect        
+ * Rumble effect.
+ * @param strength      
+ * Strength of rumble effect.
  *
  * @return true if the rumble state has been successfully set
  **/
-bool input_driver_set_rumble_state(unsigned port,
-      enum retro_rumble_effect effect, uint16_t strength);
+bool input_driver_set_rumble(
+         unsigned port, unsigned joy_idx, 
+         enum retro_rumble_effect effect, uint16_t strength);
+/**
+ * Sets the rumble gain.
+ *
+ * @param gain             
+ * Rumble gain, 0-100 [%]
+ * @param input_max_users
+ * TODO/FIXME - ???
+ *
+ * @return true if the rumble gain has been successfully set
+ **/
+bool input_driver_set_rumble_gain(
+         unsigned gain,
+         unsigned input_max_users);
 
 /**
- * Sets the sensor state. Used by RETRO_ENVIRONMENT_GET_SENSOR_INTERFACE.
+ * Sets the sensor state.
  * 
  * @param port
- * @param effect  Sensor action
- * @param rate    Sensor rate update
+ * User number.
+ * @param sensors_enable
+ * TODO/FIXME - ???
+ * @param effect        
+ * Sensor action
+ * @param rate          
+ * Sensor rate update
  *
  * @return true if the sensor state has been successfully set
  **/
-bool input_sensor_set_state(unsigned port,
-      enum retro_sensor_action action, unsigned rate);
+bool input_driver_set_sensor(
+         unsigned port, bool sensors_enable,
+         enum retro_sensor_action action, unsigned rate);
 
 /**
  * Retrieves the sensor state associated with the provided port and ID. 
  * 
  * @param port
- * @param id    Sensor ID
+ * Port of the device
+ * @param sensors_enable
+ * TODO/FIXME - ???
+ * @param id            
+ * Sensor ID
  *
  * @return The current state associated with the port and ID as a float
  **/
-float input_sensor_get_input(unsigned port, unsigned id);
+float input_driver_get_sensor(
+         unsigned port, bool sensors_enable, unsigned id);
 
-/**
- * Retrieves the input driver state struct
- * 
- * @return The input state struct
- **/
-void *input_driver_get_data(void);
+uint64_t input_driver_get_capabilities(void);
 
-/**
- * Sets the input_driver_nonblock_state flag to true
- **/
-void input_driver_set_nonblock_state(void);
+bool video_driver_init_input(
+      input_driver_t *tmp,
+      settings_t *settings,
+      bool verbosity_enabled);
 
-/**
- * Sets the input_driver_nonblock_state flag to false
- **/
-void input_driver_unset_nonblock_state(void);
+bool input_driver_grab_mouse(void);
 
-/**
- * If the action is INPUT_ACTION_AXIS_THRESHOLD, return the current
- * input_driver_axis_threshold.
- * 
- * @return value of input_driver_axis_threshold or NULL for actions other than
- *          INPUT_ACTION_AXIS_THRESHOLD
-**/
-float *input_driver_get_float(enum input_action action);
-
-/**
- * If the action is INPUT_ACTION_MAX_USERS, return the current
- * input_driver_max_users.
- * 
- * @return value of input_driver_axis_threshold or NULL for actions other than
- *          INPUT_ACTION_AXIS_THRESHOLD
-**/
-unsigned *input_driver_get_uint(enum input_action action);
+bool input_driver_ungrab_mouse(void);
 
 /**
  * Get an enumerated list of all joypad driver names
@@ -362,6 +625,8 @@ const char* config_get_joypad_driver_options(void);
  * zero-length string, equivalent to calling input_joypad_init_first().
  *
  * @param ident  identifier of driver to initialize.
+ * @param data   joypad state data pointer, which can be NULL and will be
+ *               initialized by the new joypad driver, if one is found.
  *
  * @return The joypad driver if found, otherwise NULL.
  **/
@@ -401,13 +666,32 @@ const input_device_driver_t *input_joypad_init_driver(
 /**
  * Registers a newly connected pad with RetroArch.
  * 
- * @param port    Joystick number
- * @param driver  Handle for joypad driver handling joystick's input
+ * @param port    
+ * Joystick number
+ * @param driver  
+ * Handle for joypad driver handling joystick's input
  **/
 void input_pad_connect(unsigned port, input_device_driver_t *driver);
 
+/**
+ * Called by drivers when keyboard events are fired. Interfaces with the global
+ * driver struct and libretro callbacks.
+ * 
+ * @param down       
+ * Was Keycode pressed down?
+ * @param code       
+ * Keycode.
+ * @param character  
+ * Character inputted.
+ * @param mod        
+ * TODO/FIXME/???
+ **/
+void input_keyboard_event(bool down, unsigned code, uint32_t character,
+      uint16_t mod, unsigned device);
 
-/*****************************************************************************/
+input_driver_state_t *input_state_get_ptr(void);
+
+/*************************************/
 #ifdef HAVE_HID
 #include "include/hid_driver.h"
 
@@ -430,7 +714,7 @@ const hid_driver_t *input_hid_init_first(void);
  * 
  * @return Pointer to hid_data struct
  **/
-const void *hid_driver_get_data(void);
+void *hid_driver_get_data(void);
 
 /**
  * This should be called after we've invoked free() on the HID driver; the
@@ -439,48 +723,14 @@ const void *hid_driver_get_data(void);
 void hid_driver_reset_data(void);
 
 #endif /* HAVE_HID */
-/*****************************************************************************/
+/*************************************/
 
 
-/**
- * line_complete callback (when carriage return is pressed)
- *
- * @param userdata User data which will be passed to subsequent callbacks.
- * @param line      the line of input, which can be NULL.
- **/
-typedef void (*input_keyboard_line_complete_t)(void *userdata,
-      const char *line);
-
-/**
- * Callback for keypress events
- * 
- * @param userdata The user data that was passed through from the keyboard press callback.
- * @param code      keycode
- **/
-typedef bool (*input_keyboard_press_t)(void *userdata, unsigned code);
-
-struct input_keyboard_ctx_wait
-{
-   void *userdata;
-   input_keyboard_press_t cb;
-};
-
-/**
- * Called by drivers when keyboard events are fired. Interfaces with the global
- * driver struct and libretro callbacks.
- * 
- * @param down       Was Keycode pressed down?
- * @param code       Keycode.
- * @param character  Character inputted.
- * @param mod        TODO/FIXME/???
- **/
-void input_keyboard_event(bool down, unsigned code, uint32_t character,
-      uint16_t mod, unsigned device);
-
 /**
  * Set the name of the device in the specified port
  * 
  * @param port
+ * The port of the device to be assigned to
  */
 void input_config_set_device_name(unsigned port, const char *name);
 
@@ -488,14 +738,18 @@ void input_config_set_device_name(unsigned port, const char *name);
  * Set the formatted "display name" of the device in the specified port
  * 
  * @param port
+ * The port of the device to be assigned to
  */
 void input_config_set_device_display_name(unsigned port, const char *name);
+void input_config_set_mouse_display_name(unsigned port, const char *name);
 
 /**
  * Set the configuration path for the device in the specified port
  * 
  * @param port
- * @param path The path of the device config.
+ * The port of the device to be assigned to
+ * @param path 
+ * The path of the device config.
  */
 void input_config_set_device_config_path(unsigned port, const char *path);
 
@@ -503,7 +757,9 @@ void input_config_set_device_config_path(unsigned port, const char *path);
  * Set the configuration name for the device in the specified port
  * 
  * @param port
- * @param name The name of the config to set.
+ * The port of the device to be assigned to
+ * @param name 
+ * The name of the config to set.
  */
 void input_config_set_device_config_name(unsigned port, const char *name);
 
@@ -511,7 +767,9 @@ void input_config_set_device_config_name(unsigned port, const char *name);
  * Set the joypad driver for the device in the specified port
  * 
  * @param port
- * @param driver The driver to set the given port to.
+ * The port of the device to be assigned to
+ * @param driver 
+ * The driver to set the given port to.
  */
 void input_config_set_device_joypad_driver(unsigned port, const char *driver);
 
@@ -519,7 +777,9 @@ void input_config_set_device_joypad_driver(unsigned port, const char *driver);
  * Set the vendor ID (vid) for the device in the specified port
  * 
  * @param port
- * @param vid The VID to set the given device port to.
+ * The port of the device to be assigned to
+ * @param vid 
+ * The VID to set the given device port to.
  */
 void input_config_set_device_vid(unsigned port, uint16_t vid);
 
@@ -527,7 +787,9 @@ void input_config_set_device_vid(unsigned port, uint16_t vid);
  * Set the pad ID (pid) for the device in the specified port
  * 
  * @param port
- * @param pid The PID to set the given device port to.
+ * The port of the device to be assigned to
+ * @param pid 
+ * The PID to set the given device port to.
  */
 void input_config_set_device_pid(unsigned port, uint16_t pid);
 
@@ -535,7 +797,9 @@ void input_config_set_device_pid(unsigned port, uint16_t pid);
  * Sets the autoconfigured flag for the device in the specified port
  *
  * @param port
- * @param autoconfigured Whether or nor the device is configured automatically.
+ * The port of the device to be assigned to
+ * @param autoconfigured 
+ * Whether or nor the device is configured automatically.
  */
 void input_config_set_device_autoconfigured(unsigned port, bool autoconfigured);
 
@@ -543,7 +807,9 @@ void input_config_set_device_autoconfigured(unsigned port, bool autoconfigured);
  * Sets the name index number for the device in the specified port
  *
  * @param port
- * @param name_index The name index to set the device to use.
+ * The port of the device to be assigned to
+ * @param name_index 
+ * The name index to set the device to use.
  */
 void input_config_set_device_name_index(unsigned port, unsigned name_index);
 
@@ -551,18 +817,12 @@ void input_config_set_device_name_index(unsigned port, unsigned name_index);
  * Sets the device type of the specified port
  * 
  * @param port
- * @param id The device type (RETRO_DEVICE_JOYPAD, RETRO_DEVICE_MOUSE, etc)
+ * The port of the device to be assigned to
+ * @param id 
+ * The device type (RETRO_DEVICE_JOYPAD, RETRO_DEVICE_MOUSE, etc)
  */
 void input_config_set_device(unsigned port, unsigned id);
 
-/**
- * Registers a pad_connection_listener_interface with a function pointer that
- * is called when a joypad is connected. Only used by the wiiu_joypad driver.
- *
- * @param listener  a struct that implements pad_connection_listener_interface
- */
-void set_connection_listener(pad_connection_listener_t *listener);
-
 /* Clear input_device_info */
 void input_config_clear_device_name(unsigned port);
 void input_config_clear_device_display_name(unsigned port);
@@ -579,6 +839,7 @@ unsigned input_config_get_device(unsigned port);
 /* Get input_device_info */
 const char *input_config_get_device_name(unsigned port);
 const char *input_config_get_device_display_name(unsigned port);
+const char *input_config_get_mouse_display_name(unsigned port);
 const char *input_config_get_device_config_path(unsigned port);
 const char *input_config_get_device_config_name(unsigned port);
 const char *input_config_get_device_joypad_driver(unsigned port);
@@ -587,6 +848,7 @@ const char *input_config_get_device_joypad_driver(unsigned port);
  * Retrieves the vendor id (vid) of a connected controller
  * 
  * @param port
+ * The port of the device
  *
  * @return the vendor id VID of the device
  */
@@ -596,6 +858,7 @@ uint16_t input_config_get_device_vid(unsigned port);
  * Retrieves the pad id (pad) of a connected controller
  * 
  * @param port
+ * The port of the device
  *
  * @return the port id PID of the device
  */
@@ -605,6 +868,7 @@ uint16_t input_config_get_device_pid(unsigned port);
  * Returns the value of the autoconfigured flag for the specified device
  *
  * @param port
+ * The port of the device
  *
  * @return the autoconfigured flag
  */
@@ -614,6 +878,7 @@ bool input_config_get_device_autoconfigured(unsigned port);
  * Get the name index number for the device in this port
  * 
  * @param port
+ * The port of the device
  *
  * @return the name index for this device
  */
@@ -630,6 +895,7 @@ unsigned input_config_get_device_name_index(unsigned port);
  * low-level access is not required.
  * 
  * @param port
+ * The port of the device
  *
  * @return a pointer to the device name on the specified port
  */
@@ -643,105 +909,169 @@ char *input_config_get_device_name_ptr(unsigned port);
  * low-level access is not required.
  * 
  * @param port
+ * The port of the device
  *
  * @return the size of the device name on the specified port
  */
 size_t input_config_get_device_name_size(unsigned port);
 
-/*****************************************************************************/
+bool input_driver_button_combo(
+      unsigned mode,
+      retro_time_t current_time,
+      input_bits_t* p_input);
+
+bool input_driver_find_driver(
+      settings_t *settings,
+      const char *prefix,
+      bool verbosity_enabled);
+
+void input_keyboard_line_append(
+      struct input_keyboard_line *keyboard_line,
+      const char *word, size_t len);
 
 /**
- * Save the current keybinds on a port to the config file.
- * 
- * @param conf  pointer to config file object
- * @param user  user number (ie port - TODO: change to port nomenclature)
- */
-void input_config_save_keybinds_user(void *data, unsigned user);
+ * input_keyboard_start_line:
+ * @userdata                 : Userdata.
+ * @cb                       : Line complete callback function.
+ *
+ * Sets function pointer for keyboard line handle.
+ *
+ * The underlying buffer can be reallocated at any time
+ * (or be NULL), but the pointer to it remains constant
+ * throughout the objects lifetime.
+ *
+ * Returns: underlying buffer of the keyboard line.
+ **/
+const char **input_keyboard_start_line(
+      void *userdata,
+      struct input_keyboard_line *keyboard_line,
+      input_keyboard_line_complete_t cb);
+
+#if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
+input_remote_t *input_driver_init_remote(
+      settings_t *settings,
+      unsigned num_active_users);
+
+void input_remote_free(input_remote_t *handle, unsigned max_users);
+#endif
+
+void input_game_focus_free(void);
+
+void input_config_get_bind_string_joyaxis(
+      bool input_descriptor_label_show,
+      char *buf, const char *prefix,
+      const struct retro_keybind *bind, size_t size);
+
+void input_config_get_bind_string_joykey(
+      bool input_descriptor_label_show,
+      char *buf, const char *prefix,
+      const struct retro_keybind *bind, size_t size);
+
+int16_t input_state_internal(unsigned port, unsigned device,
+      unsigned idx, unsigned id);
+
+bool input_key_pressed(int key, bool keyboard_pressed);
+
+bool input_set_rumble_state(unsigned port,
+      enum retro_rumble_effect effect, uint16_t strength);
+
+bool input_set_rumble_gain(unsigned gain);
+
+float input_get_sensor_state(unsigned port, unsigned id);
+
+bool input_set_sensor_state(unsigned port,
+      enum retro_sensor_action action, unsigned rate);
+
+void *input_driver_init_wrap(input_driver_t *input, const char *name);
 
 const struct retro_keybind *input_config_get_bind_auto(unsigned port, unsigned id);
 
-/**
- * Save a key binding to the config file.
- * 
- * @param conf    pointer to config file object
- * @param prefix  prefix name of keybind
- * @param base    base name of keybind
- * @param bind    pointer to key binding object
- * @param kb      save keyboard binds
- */
-void input_config_save_keybind(void *data, const char *prefix,
-      const char *base, const struct retro_keybind *bind,
-      bool save_empty);
-
 void input_config_reset_autoconfig_binds(unsigned port);
+
 void input_config_reset(void);
 
-#if defined(ANDROID)
-#define DEFAULT_MAX_PADS 8
-#define ANDROID_KEYBOARD_PORT DEFAULT_MAX_PADS
-#elif defined(_3DS)
-#define DEFAULT_MAX_PADS 1
-#elif defined(SWITCH) || defined(HAVE_LIBNX)
-#define DEFAULT_MAX_PADS 8
-#elif defined(WIIU)
-#ifdef WIIU_HID
-#define DEFAULT_MAX_PADS 16
-#else
-#define DEFAULT_MAX_PADS 5
-#endif /* WIIU_HID */
-#elif defined(DJGPP)
-#define DEFAULT_MAX_PADS 1
-#define DOS_KEYBOARD_PORT DEFAULT_MAX_PADS
-#elif defined(XENON)
-#define DEFAULT_MAX_PADS 4
-#elif defined(VITA) || defined(SN_TARGET_PSP2)
-#define DEFAULT_MAX_PADS 4
-#elif defined(PSP)
-#define DEFAULT_MAX_PADS 1
-#elif defined(PS2)
-#define DEFAULT_MAX_PADS 8
-#elif defined(GEKKO) || defined(HW_RVL)
-#define DEFAULT_MAX_PADS 4
-#elif defined(HAVE_ODROIDGO2)
-#define DEFAULT_MAX_PADS 1
-#elif defined(__linux__) || (defined(BSD) && !defined(__MACH__))
-#define DEFAULT_MAX_PADS 8
-#elif defined(__QNX__)
-#define DEFAULT_MAX_PADS 8
-#elif defined(__PS3__)
-#define DEFAULT_MAX_PADS 7
-#elif defined(_XBOX)
-#define DEFAULT_MAX_PADS 4
-#elif defined(HAVE_XINPUT) && !defined(HAVE_DINPUT)
-#define DEFAULT_MAX_PADS 4
-#elif defined(DINGUX)
-#define DEFAULT_MAX_PADS 2
-#else
-#define DEFAULT_MAX_PADS 16
-#endif /* defined(ANDROID) */
+const char *joypad_driver_name(unsigned i);
 
-extern input_device_driver_t dinput_joypad;
-extern input_device_driver_t linuxraw_joypad;
-extern input_device_driver_t parport_joypad;
-extern input_device_driver_t udev_joypad;
-extern input_device_driver_t xinput_joypad;
-extern input_device_driver_t sdl_joypad;
-extern input_device_driver_t sdl_dingux_joypad;
-extern input_device_driver_t ps4_joypad;
-extern input_device_driver_t ps3_joypad;
-extern input_device_driver_t psp_joypad;
-extern input_device_driver_t ps2_joypad;
-extern input_device_driver_t ctr_joypad;
-extern input_device_driver_t switch_joypad;
-extern input_device_driver_t xdk_joypad;
-extern input_device_driver_t gx_joypad;
-extern input_device_driver_t wiiu_joypad;
-extern input_device_driver_t hid_joypad;
-extern input_device_driver_t android_joypad;
-extern input_device_driver_t qnx_joypad;
-extern input_device_driver_t mfi_joypad;
-extern input_device_driver_t dos_joypad;
-extern input_device_driver_t rwebpad_joypad;
+void joypad_driver_reinit(void *data, const char *joypad_driver_name);
+
+#ifdef HAVE_COMMAND
+void input_driver_init_command(
+      input_driver_state_t *input_st,
+      settings_t *settings);
+
+void input_driver_deinit_command(input_driver_state_t *input_st);
+#endif
+
+#ifdef HAVE_OVERLAY
+void input_overlay_deinit(void);
+
+void input_overlay_init(void);
+#endif
+
+#ifdef HAVE_BSV_MOVIE
+void bsv_movie_frame_rewind(void);
+void bsv_movie_next_frame(input_driver_state_t *input_st);
+void bsv_movie_finish_rewind(input_driver_state_t *input_st);
+void bsv_movie_deinit(input_driver_state_t *input_st);
+
+bool movie_start_playback(input_driver_state_t *input_st, char *path);
+bool movie_start_record(input_driver_state_t *input_st, char *path);
+bool movie_stop_playback(input_driver_state_t *input_st);
+bool movie_stop_record(input_driver_state_t *input_st);
+bool movie_stop(input_driver_state_t *input_st);
+
+size_t replay_get_serialize_size(void);
+bool replay_get_serialized_data(void* buffer);
+bool replay_set_serialized_data(void* buffer);
+#endif
+
+/**
+ * input_poll:
+ *
+ * Input polling callback function.
+ **/
+void input_driver_poll(void);
+
+/**
+ * input_state_wrapper:
+ * @port                 : user number.
+ * @device               : device identifier of user.
+ * @idx                  : index value of user.
+ * @id                   : identifier of key pressed by user.
+ *
+ * Input state callback function.
+ *
+ * Returns: Non-zero if the given key (identified by @id)
+ * was pressed by the user (assigned to @port).
+ **/
+int16_t input_driver_state_wrapper(unsigned port, unsigned device,
+      unsigned idx, unsigned id);
+
+void input_driver_collect_system_input(input_driver_state_t *input_st,
+      settings_t *settings, input_bits_t *current_bits);
+
+/**
+ * input_keyboard_event:
+ * @down                     : Keycode was pressed down?
+ * @code                     : Keycode.
+ * @character                : Character inputted.
+ * @mod                      : TODO/FIXME: ???
+ *
+ * Keyboard event utils. Called by drivers when keyboard events 
+ * are fired.
+ * This interfaces with the global system driver struct 
+ * and libretro callbacks.
+ **/
+void input_keyboard_event(bool down, unsigned code,
+      uint32_t character, uint16_t mod, unsigned device);
+
+extern const unsigned input_config_bind_order[24];
+
+extern input_device_driver_t *joypad_drivers[];
+extern input_driver_t *input_drivers[];
+#ifdef HAVE_HID
+extern hid_driver_t *hid_drivers[];
+#endif
 
 extern input_driver_t input_android;
 extern input_driver_t input_sdl;
@@ -768,38 +1098,37 @@ extern input_driver_t input_dos;
 extern input_driver_t input_winraw;
 extern input_driver_t input_wayland;
 
+extern input_device_driver_t dinput_joypad;
+extern input_device_driver_t linuxraw_joypad;
+extern input_device_driver_t parport_joypad;
+extern input_device_driver_t udev_joypad;
+extern input_device_driver_t xinput_joypad;
+extern input_device_driver_t sdl_joypad;
+extern input_device_driver_t sdl_dingux_joypad;
+extern input_device_driver_t ps4_joypad;
+extern input_device_driver_t ps3_joypad;
+extern input_device_driver_t psp_joypad;
+extern input_device_driver_t ps2_joypad;
+extern input_device_driver_t ctr_joypad;
+extern input_device_driver_t switch_joypad;
+extern input_device_driver_t xdk_joypad;
+extern input_device_driver_t gx_joypad;
+extern input_device_driver_t wiiu_joypad;
+extern input_device_driver_t hid_joypad;
+extern input_device_driver_t android_joypad;
+extern input_device_driver_t qnx_joypad;
+extern input_device_driver_t mfi_joypad;
+extern input_device_driver_t dos_joypad;
+extern input_device_driver_t rwebpad_joypad;
+
 #ifdef HAVE_HID
 extern hid_driver_t iohidmanager_hid;
 extern hid_driver_t btstack_hid;
 extern hid_driver_t libusb_hid;
 extern hid_driver_t wiiusb_hid;
+extern hid_driver_t wiiu_hid;
 #endif /* HAVE_HID */
 
-typedef struct menu_input_ctx_line
-{
-   const char *label;
-   const char *label_setting;
-   unsigned type;
-   unsigned idx;
-   input_keyboard_line_complete_t cb;
-} menu_input_ctx_line_t;
-
-const char *menu_input_dialog_get_label_setting_buffer(void);
-
-const char *menu_input_dialog_get_label_buffer(void);
-
-const char *menu_input_dialog_get_buffer(void);
-
-unsigned menu_input_dialog_get_kb_idx(void);
-
-bool menu_input_dialog_start_search(void);
-
-bool menu_input_dialog_get_display_kb(void);
-
-bool menu_input_dialog_start(menu_input_ctx_line_t *line);
-
-void menu_input_dialog_end(void);
-
 RETRO_END_DECLS
 
 #endif /* __INPUT_DRIVER__H */
