@@ -87,6 +87,8 @@ MSG_HASH( /* FIXME Is a specific image format used? Is it determined automatical
    MENU_ENUM_SUBLABEL_DUMP_DISC,
    "Dympio’r disg corfforol cyfryngau i’r storfa fewnol. Bydd yn cadw fel ffeil delwedd."
    )
+#ifdef HAVE_LAKKA
+#endif
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_PLAYLISTS_TAB,
    "Rhestri Chwarae"
@@ -445,26 +447,14 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_CORE_INFO_REQUIRED_HW_API,
    "API Graffeg Angenrheidiol"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_CORE_INFO_SAVESTATE_DISABLED,
+   "Dim"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_CORE_INFO_FIRMWARE,
    "Cadarnwedd"
    )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_MISSING,
-   "Ar goll"
-   )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_PRESENT,
-   "Presennol"
-   )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_OPTIONAL,
-   "Dewisol"
-   )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_REQUIRED,
-   "Angenrheidiol"
-   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_CORE_LOCK,
    "Cloi Craidd wedi'i osod"
@@ -528,17 +518,9 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_CPU_FEATURES,
    "Nodweddion CPU"
    )
-MSG_HASH( /* FIXME Colon should be handled in menu_display.c like the rest */
-   MENU_ENUM_LABEL_VALUE_CPU_ARCHITECTURE,
-   "Pensaernïaeth CPU:"
-   )
-MSG_HASH( /* FIXME Colon should be handled in menu_display.c like the rest */
-   MENU_ENUM_LABEL_VALUE_CPU_CORES,
-   "Creiddiau CPU:"
-   )
 MSG_HASH(
-   MENU_ENUM_SUBLABEL_CPU_CORES,
-   "Nifer o greiddiau sydd gan y CPU."
+   MENU_ENUM_LABEL_VALUE_CPU_ARCHITECTURE,
+   "Pensaernïaeth CPU"
    )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_FRONTEND_IDENTIFIER,
@@ -776,10 +758,6 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_NETPLAY_SUPPORT,
    "Cymorth Netplay (Cymar wrth gymar)"
    )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_PYTHON_SUPPORT,
-   "Cefnogaeth Python (Cymorth Sgript Shaders)"
-   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_V4L2_SUPPORT,
    "Cymorth Video4Linux2"
@@ -806,6 +784,10 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_RDB_ENTRY_DESCRIPTION,
    "Disgrifiad"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_RDB_ENTRY_ACHIEVEMENTS,
+   "Llwyddiannau"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_RDB_ENTRY_PUBLISHER,
    "Cyhoeddwr"
@@ -842,14 +824,6 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_RDB_ENTRY_EDGE_MAGAZINE_ISSUE,
    "Rhifyn Cylchgrawn Edge"
    )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_RDB_ENTRY_RELEASE_MONTH,
-   "Mis rhyddhau"
-   )
-MSG_HASH(
-   MENU_ENUM_LABEL_VALUE_RDB_ENTRY_RELEASE_YEAR,
-   "Blwyddyn rhyddhau"
-   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_RDB_ENTRY_BBFC_RATING,
    "Asesiad BBFC"
@@ -906,6 +880,10 @@ MSG_HASH(
 
 /* Settings */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_INPUT_SETTINGS,
+   "Mewnbwn"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_CORE_SETTINGS,
    "Craidd"
@@ -1007,6 +985,11 @@ MSG_HASH(
    "Newid cyfeirlyfrau diofyn lle mae ffeiliau wedi'u lleoli."
    )
 
+/* Core option category placeholders for icons */
+
+#ifdef HAVE_MIST
+#endif
+
 /* Settings > Drivers */
 
 MSG_HASH(
@@ -1021,6 +1004,8 @@ MSG_HASH(
 /* Settings > Video */
 
 #if defined(DINGUX)
+#if defined(RS90) || defined(MIYOO)
+#endif
 #endif
 
 /* Settings > Video > CRT SwitchRes */
@@ -1028,6 +1013,8 @@ MSG_HASH(
 
 /* Settings > Video > Output */
 
+#if defined (WIIU)
+#endif
 #if defined(DINGUX) && defined(DINGUX_BETA)
 #endif
 
@@ -1042,6 +1029,9 @@ MSG_HASH(
 #if defined(DINGUX)
 #endif
 
+/* Settings > Video > HDR */
+
+
 /* Settings > Video > Synchronization */
 
 
@@ -1063,6 +1053,10 @@ MSG_HASH(
 
 /* Settings > Audio > MIDI */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_MIDI_INPUT,
+   "Mewnbwn"
+   )
 
 /* Settings > Audio > Mixer Settings > Mixer Stream */
 
@@ -1082,6 +1076,8 @@ MSG_HASH(
 
 #if defined(HAVE_DINPUT) || defined(HAVE_WINRAWINPUT)
 #endif
+#ifdef ANDROID
+#endif
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_INPUT_AUTO_GAME_FOCUS_OFF,
    "BANT"
@@ -1093,8 +1089,6 @@ MSG_HASH(
 
 /* Settings > Input > Haptic Feedback/Vibration */
 
-#if defined(DINGUX) && defined(HAVE_LIBSHAKE)
-#endif
 
 /* Settings > Input > Menu Controls */
 
@@ -1103,9 +1097,23 @@ MSG_HASH(
 
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_INPUT_META_QUIT_KEY,
-   "Gadael RetroArch"
+   "Cau"
    )
 
+
+
+
+
+
+
+
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_INPUT_META_AI_SERVICE,
+   "Gwasanaeth AI"
+   )
+
+
+
 /* Settings > Input > Port # Controls */
 
 
@@ -1116,7 +1124,15 @@ MSG_HASH(
 
 #ifndef HAVE_DYNAMIC
 #endif
+#ifdef HAVE_MIST
 
+
+
+
+
+
+
+#endif
 /* Settings > Configuration */
 
 
@@ -1140,12 +1156,17 @@ MSG_HASH(
 
 /* Settings > Recording */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_VIDEO_STREAMING_MODE_LOCAL,
+   "Lleol"
+   )
 
 /* Settings > On-Screen Display */
 
 
 /* Settings > On-Screen Display > On-Screen Overlay */
 
+
 #if defined(ANDROID)
 #endif
 
@@ -1157,6 +1178,8 @@ MSG_HASH(
 
 /* Settings > User Interface */
 
+#ifdef _3DS
+#endif
 
 /* Settings > User Interface > Menu Item Visibility */
 
@@ -1168,10 +1191,16 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_SETTINGS_VIEWS_SETTINGS,
    "Gosodiadau"
    )
+#ifdef HAVE_LAKKA
+#endif
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_MENU_ADD_CONTENT_ENTRY_DISPLAY_MAIN_TAB,
    "Prif Ddewislen"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_SHOW_CONTENTLESS_CORES_ALL,
+   "Popeth"
+   )
 
 /* Settings > User Interface > Menu Item Visibility > Quick Menu */
 
@@ -1179,6 +1208,7 @@ MSG_HASH(
 /* Settings > User Interface > Views > Settings */
 
 
+
 /* Settings > User Interface > Appearance */
 
 
@@ -1192,6 +1222,16 @@ MSG_HASH(
 
 /* Settings > Achievements */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_CHEEVOS_ENABLE,
+   "Llwyddiannau"
+   )
+
+/* Settings > Achievements > Appearance */
+
+
+/* Settings > Achievements > Visibility */
+
 
 /* Settings > Network */
 
@@ -1245,6 +1285,13 @@ MSG_HASH(
    "Rhestri Chwarae"
    )
 
+#ifdef HAVE_MIST
+/* Settings > Steam */
+
+
+
+#endif
+
 /* Music */
 
 /* Music > Quick Menu */
@@ -1268,6 +1315,10 @@ MSG_HASH(
 
 /* Import Content > Manual Scan */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_MANUAL_CONTENT_SCAN_SYSTEM_NAME,
+   "Enw System"
+   )
 
 /* Explore tab */
 MSG_HASH(
@@ -1302,6 +1353,10 @@ MSG_HASH(
 
 /* Quick Menu */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_ACHIEVEMENT_LIST,
+   "Llwyddiannau"
+   )
 
 /* Quick Menu > Options */
 
@@ -1314,7 +1369,10 @@ MSG_HASH(
 /* Quick Menu > Controls */
 
 
-/* Quick Menu > Controls > Load Remap File */
+/* Quick Menu > Controls > Manage Remap Files */
+
+
+/* Quick Menu > Controls > Manage Remap Files > Load Remap File */
 
 
 /* Quick Menu > Cheats */
@@ -1331,6 +1389,10 @@ MSG_HASH(
 
 /* Quick Menu > Cheats > Cheat Details */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_CHEAT_DESC,
+   "Disgrifiad"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_CHEAT_TYPE,
    "Math"
@@ -1395,6 +1457,10 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_NONE,
    "Dim"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_UNKNOWN,
+   "Anhysybys"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_RUMBLE_PORT_16,
    "Popeth"
@@ -1425,6 +1491,10 @@ MSG_HASH(
 
 /* RGUI: Settings Options */
 
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_RGUI_UPSCALE_NONE,
+   "Dim"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_RGUI_ASPECT_RATIO_LOCK_NONE,
    "BANT"
@@ -1491,6 +1561,10 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_QT_LOAD_CORE,
    "Llwytho Craidd"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_QT_NAME,
+   "Enw"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_QT_TAB_PLAYLISTS,
    "Rhestri Chwarae"
@@ -1511,6 +1585,10 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_QT_STOP,
    "Stopio"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_QT_PLAYLIST_ENTRY_NAME,
+   "Enw:"
+   )
 MSG_HASH(
    MENU_ENUM_LABEL_VALUE_QT_PLAYLIST_ENTRY_CORE,
    "Craidd:"
@@ -1541,6 +1619,10 @@ MSG_HASH(
    MENU_ENUM_LABEL_VALUE_HELP,
    "Cymorth"
    )
+MSG_HASH(
+   MENU_ENUM_LABEL_VALUE_CHEEVOS_DESCRIPTION,
+   "Disgrifiad"
+   )
 
 /* Unused (Needs Confirmation) */
 
@@ -1562,6 +1644,7 @@ MSG_HASH(
 
 /* Notifications */
 
+
 MSG_HASH(
    MSG_TO,
    "i"
@@ -1574,18 +1657,6 @@ MSG_HASH(
    MSG_VALUE_REBOOTING,
    "Yn Ailgychwyn..."
    )
-MSG_HASH(
-   MSG_VIRTUAL_DISK_TRAY_CLOSE,
-   "cau"
-   )
-MSG_HASH(
-   MSG_FAILED,
-   "wedi methu"
-   )
-MSG_HASH(
-   MSG_SUCCEEDED,
-   "wedi llwyddo"
-   )
 MSG_HASH(
    MSG_ENABLING_WIRELESS,
    "Yn galluogi Wi-Fi..."
@@ -1652,10 +1723,6 @@ MSG_HASH(
    MSG_LOCAL,
    "Lleol"
    )
-MSG_HASH(
-   MSG_READ_WRITE,
-   "Darllen/Ysgrifennu"
-   )
 
 #ifdef HAVE_LAKKA_SWITCH
 #endif
@@ -1668,7 +1735,7 @@ MSG_HASH(
 #ifdef HAVE_ODROIDGO2
 #else
 #endif
-#if defined(_3DS)
+#ifdef _3DS
 #endif
 #ifdef HAVE_QT
 #endif
