@@ -28,7 +28,7 @@
 #include "../configuration.h"
 #include "../file_path_special.h"
 #include "../list_special.h"
-#include "../retroarch.h"
+#include "../verbosity.h"
 #include "../input/input_driver.h"
 #include "../input/input_remapping.h"
 
@@ -37,6 +37,16 @@
 #include "../input/include/blissbox.h"
 #endif
 
+#ifdef HAVE_MENU
+#include "../menu/menu_driver.h"
+#endif
+
+enum autoconfig_handle_flags
+{
+   AUTOCONF_FLAG_AUTOCONFIG_ENABLED     = (1 << 0),
+   AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS = (1 << 1)
+};
+
 typedef struct
 {
    char *dir_autoconfig;
@@ -44,8 +54,7 @@ typedef struct
    config_file_t *autoconfig_file;
    unsigned port;
    input_device_info_t device_info; /* unsigned alignment */
-   bool autoconfig_enabled;
-   bool suppress_notifcations;
+   uint8_t flags;
 } autoconfig_handle_t;
 
 /*********************/
@@ -82,13 +91,8 @@ static void free_autoconfig_handle(autoconfig_handle_t *autoconfig_handle)
 static void input_autoconfigure_free(retro_task_t *task)
 {
    autoconfig_handle_t *autoconfig_handle = NULL;
-
-   if (!task)
-      return;
-
-   autoconfig_handle = (autoconfig_handle_t*)task->state;
-
-   free_autoconfig_handle(autoconfig_handle);
+   if (task && (autoconfig_handle = (autoconfig_handle_t*)task->state))
+      free_autoconfig_handle(autoconfig_handle);
 }
 
 /******************************/
@@ -244,9 +248,7 @@ static bool input_autoconfigure_scan_config_files_external(
          continue;
 
       /* Load autoconfig file */
-      config = config_file_new_from_path_to_string(config_file_path);
-
-      if (!config)
+      if (!(config = config_file_new_from_path_to_string(config_file_path)))
          continue;
 
       /* Check for a match */
@@ -369,9 +371,7 @@ static void cb_input_autoconfigure_connect(
    if (!task)
       return;
 
-   autoconfig_handle = (autoconfig_handle_t*)task->state;
-
-   if (!autoconfig_handle)
+   if (!(autoconfig_handle = (autoconfig_handle_t*)task->state))
       return;
 
    /* Use local copy of port index for brevity... */
@@ -445,7 +445,7 @@ static void input_autoconfigure_connect_handler(retro_task_t *task)
    autoconfig_handle_t *autoconfig_handle = NULL;
    bool match_found                       = false;
    const char *device_display_name        = NULL;
-   char task_title[PATH_MAX_LENGTH];
+   char task_title[NAME_MAX_LENGTH + 16];
 
    task_title[0] = '\0';
 
@@ -454,9 +454,9 @@ static void input_autoconfigure_connect_handler(retro_task_t *task)
 
    autoconfig_handle = (autoconfig_handle_t*)task->state;
 
-   if (!autoconfig_handle ||
-       string_is_empty(autoconfig_handle->device_info.name) ||
-       !autoconfig_handle->autoconfig_enabled)
+   if (   !autoconfig_handle
+       || string_is_empty(autoconfig_handle->device_info.name)
+       || !(autoconfig_handle->flags & AUTOCONF_FLAG_AUTOCONFIG_ENABLED))
       goto task_finished;
 
    /* Annoyingly, we have to scan all the autoconfig
@@ -469,10 +469,8 @@ static void input_autoconfigure_connect_handler(retro_task_t *task)
    /* Scan in order of preference:
     * - External autoconfig files
     * - Internal autoconfig definitions */
-   match_found = input_autoconfigure_scan_config_files_external(
-         autoconfig_handle);
-
-   if (!match_found)
+   if (!(match_found = input_autoconfigure_scan_config_files_external(
+         autoconfig_handle)))
       match_found = input_autoconfigure_scan_config_files_internal(
          autoconfig_handle);
 
@@ -536,33 +534,42 @@ static void input_autoconfigure_connect_handler(retro_task_t *task)
       if (match_found)
       {
          /* A valid autoconfig was applied */
-         if (!autoconfig_handle->suppress_notifcations)
-            snprintf(task_title, sizeof(task_title), "%s %s #%u",
+         if (!(autoconfig_handle->flags & AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS))
+         {
+            snprintf(task_title, sizeof(task_title),
+                  msg_hash_to_str(MSG_DEVICE_CONFIGURED_IN_PORT_NR),
                   device_display_name,
-                  msg_hash_to_str(MSG_DEVICE_CONFIGURED_IN_PORT),
                   autoconfig_handle->port + 1);
+         }
       }
       /* Device is autoconfigured, but a (most likely
        * incorrect) fallback definition was used... */
       else
-         snprintf(task_title, sizeof(task_title), "%s (%u/%u) %s",
-               device_display_name,
-               autoconfig_handle->device_info.vid,
-               autoconfig_handle->device_info.pid,
-               msg_hash_to_str(MSG_DEVICE_NOT_CONFIGURED_FALLBACK));
+      {
+         snprintf(task_title, sizeof(task_title),
+                  msg_hash_to_str(MSG_DEVICE_NOT_CONFIGURED_FALLBACK_NR),
+                  device_display_name,
+                  autoconfig_handle->device_info.vid,
+                  autoconfig_handle->device_info.pid);
+      }
    }
    /* Autoconfig failed */
    else
-      snprintf(task_title, sizeof(task_title), "%s (%u/%u) %s",
-            device_display_name,
-            autoconfig_handle->device_info.vid,
-            autoconfig_handle->device_info.pid,
-            msg_hash_to_str(MSG_DEVICE_NOT_CONFIGURED));
+   {
+         snprintf(task_title, sizeof(task_title),
+                  msg_hash_to_str(MSG_DEVICE_NOT_CONFIGURED_NR),
+                  device_display_name,
+                  autoconfig_handle->device_info.vid,
+                  autoconfig_handle->device_info.pid);
+   }
 
    /* Update task title */
    task_free_title(task);
    if (!string_is_empty(task_title))
+   {
       task_set_title(task, strdup(task_title));
+      RARCH_LOG("[Autoconf]: %s.\n", task_title);
+   }
 
 task_finished:
 
@@ -589,7 +596,7 @@ static bool autoconfigure_connect_finder(retro_task_t *task, void *user_data)
    return (*port == autoconfig_handle->port);
 }
 
-void input_autoconfigure_connect(
+bool input_autoconfigure_connect(
       const char *name,
       const char *display_name,
       const char *driver,
@@ -621,9 +628,8 @@ void input_autoconfigure_connect(
       goto error;
 
    /* Configure handle */
-   autoconfig_handle = (autoconfig_handle_t*)malloc(sizeof(autoconfig_handle_t));
-
-   if (!autoconfig_handle)
+   if (!(autoconfig_handle = (autoconfig_handle_t*)
+            malloc(sizeof(autoconfig_handle_t))))
       goto error;
 
    autoconfig_handle->port                         = port;
@@ -636,8 +642,10 @@ void input_autoconfigure_connect(
    autoconfig_handle->device_info.joypad_driver[0] = '\0';
    autoconfig_handle->device_info.autoconfigured   = false;
    autoconfig_handle->device_info.name_index       = 0;
-   autoconfig_handle->autoconfig_enabled           = autoconfig_enabled;
-   autoconfig_handle->suppress_notifcations        = !notification_show_autoconfig;
+   if (autoconfig_enabled)
+      autoconfig_handle->flags |= AUTOCONF_FLAG_AUTOCONFIG_ENABLED;
+   if (!notification_show_autoconfig)
+      autoconfig_handle->flags |= AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS;
    autoconfig_handle->dir_autoconfig               = NULL;
    autoconfig_handle->dir_driver_autoconfig        = NULL;
    autoconfig_handle->autoconfig_file              = NULL;
@@ -650,8 +658,7 @@ void input_autoconfigure_connect(
       strlcpy(autoconfig_handle->device_info.display_name, display_name,
             sizeof(autoconfig_handle->device_info.display_name));
 
-   driver_valid = !string_is_empty(driver);
-   if (driver_valid)
+   if ((driver_valid = !string_is_empty(driver)))
       strlcpy(autoconfig_handle->device_info.joypad_driver,
             driver, sizeof(autoconfig_handle->device_info.joypad_driver));
 
@@ -669,10 +676,9 @@ void input_autoconfigure_connect(
       if (driver_valid)
       {
          char dir_driver_autoconfig[PATH_MAX_LENGTH];
-         dir_driver_autoconfig[0] = '\0';
-
          /* Generate driver-specific autoconfig directory */
-         fill_pathname_join(dir_driver_autoconfig, dir_autoconfig,
+         fill_pathname_join_special(dir_driver_autoconfig,
+               dir_autoconfig,
                autoconfig_handle->device_info.joypad_driver,
                sizeof(dir_driver_autoconfig));
 
@@ -698,8 +704,8 @@ void input_autoconfigure_connect(
     * task status messages
     * > Can skip this check if autoconfig notifications
     *   have been disabled by the user */
-   if (!autoconfig_handle->suppress_notifcations &&
-       !string_is_empty(autoconfig_handle->device_info.name))
+   if (   !(autoconfig_handle->flags & AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS)
+       && !string_is_empty(autoconfig_handle->device_info.name))
    {
       const char *last_device_name = input_config_get_device_name(port);
       uint16_t last_vid            = input_config_get_device_vid(port);
@@ -712,7 +718,7 @@ void input_autoconfigure_connect(
           (autoconfig_handle->device_info.vid == last_vid) &&
           (autoconfig_handle->device_info.pid == last_pid) &&
           last_autoconfigured)
-         autoconfig_handle->suppress_notifcations = true;
+         autoconfig_handle->flags |= AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS;
    }
 
    /* Configure task */
@@ -730,7 +736,7 @@ void input_autoconfigure_connect(
 
    task_queue_push(task);
 
-   return;
+   return true;
 
 error:
 
@@ -741,7 +747,7 @@ error:
    }
 
    free_autoconfig_handle(autoconfig_handle);
-   autoconfig_handle = NULL;
+   return false;
 }
 
 /****************************/
@@ -758,9 +764,7 @@ static void cb_input_autoconfigure_disconnect(
    if (!task)
       return;
 
-   autoconfig_handle = (autoconfig_handle_t*)task->state;
-
-   if (!autoconfig_handle)
+   if (!(autoconfig_handle = (autoconfig_handle_t*)task->state))
       return;
 
    /* Use local copy of port index for brevity... */
@@ -782,32 +786,35 @@ static void cb_input_autoconfigure_disconnect(
 static void input_autoconfigure_disconnect_handler(retro_task_t *task)
 {
    autoconfig_handle_t *autoconfig_handle = NULL;
-   char task_title[PATH_MAX_LENGTH];
+   const char *device_display_name        = NULL;
+   char task_title[NAME_MAX_LENGTH + 16];
 
    task_title[0] = '\0';
 
    if (!task)
       goto task_finished;
 
-   autoconfig_handle = (autoconfig_handle_t*)task->state;
-
-   if (!autoconfig_handle)
+   if (!(autoconfig_handle = (autoconfig_handle_t*)task->state))
       goto task_finished;
 
+   /* Get display name for task status message */
+   device_display_name = autoconfig_handle->device_info.display_name;
+   if (string_is_empty(device_display_name))
+      device_display_name = autoconfig_handle->device_info.name;
+   if (string_is_empty(device_display_name))
+      device_display_name = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NOT_AVAILABLE);
+
    /* Set task title */
-   if (!string_is_empty(autoconfig_handle->device_info.name))
-      snprintf(task_title, sizeof(task_title), "%s #%u (%s)",
-            msg_hash_to_str(MSG_DEVICE_DISCONNECTED_FROM_PORT),
-            autoconfig_handle->port + 1,
-            autoconfig_handle->device_info.name);
-   else
-      snprintf(task_title, sizeof(task_title), "%s #%u",
-            msg_hash_to_str(MSG_DEVICE_DISCONNECTED_FROM_PORT),
-            autoconfig_handle->port + 1);
+   snprintf(task_title, sizeof(task_title),
+         msg_hash_to_str(MSG_DEVICE_DISCONNECTED_FROM_PORT_NR),
+         device_display_name,
+         autoconfig_handle->port + 1);
 
    task_free_title(task);
-   if (!autoconfig_handle->suppress_notifcations)
+   if (!(autoconfig_handle->flags & AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS))
       task_set_title(task, strdup(task_title));
+   if (!string_is_empty(task_title))
+      RARCH_LOG("[Autoconf]: %s.\n", task_title);
 
 task_finished:
 
@@ -826,8 +833,7 @@ static bool autoconfigure_disconnect_finder(retro_task_t *task, void *user_data)
    if (task->handler != input_autoconfigure_disconnect_handler)
       return false;
 
-   autoconfig_handle = (autoconfig_handle_t*)task->state;
-   if (!autoconfig_handle)
+   if (!(autoconfig_handle = (autoconfig_handle_t*)task->state))
       return false;
 
    port = (unsigned*)user_data;
@@ -849,8 +855,10 @@ bool input_autoconfigure_disconnect(unsigned port, const char *name)
    autoconfig_handle_t *autoconfig_handle = NULL;
    task_finder_data_t find_data;
    settings_t *settings                   = config_get_ptr();
-   bool notification_show_autoconfig      = settings ?
-         settings->bools.notification_show_autoconfig : true;
+   input_driver_state_t *input_st         = input_state_get_ptr();
+   bool notification_show_autoconfig      = settings ? settings->bools.notification_show_autoconfig : true;
+   bool pause_on_disconnect               = settings ? settings->bools.pause_on_disconnect : true;
+   bool core_is_running                   = runloop_state_get_ptr()->flags & RUNLOOP_FLAG_CORE_RUNNING;
 
    if (port >= MAX_INPUT_DEVICES)
       goto error;
@@ -869,17 +877,22 @@ bool input_autoconfigure_disconnect(unsigned port, const char *name)
    if (!autoconfig_handle)
       goto error;
 
-   autoconfig_handle->port                  = port;
-   autoconfig_handle->suppress_notifcations = !notification_show_autoconfig;
+   autoconfig_handle->port      = port;
+   if (!notification_show_autoconfig)
+      autoconfig_handle->flags |= AUTOCONF_FLAG_SUPPRESS_NOTIFICATIONS;
 
-   if (!string_is_empty(name))
+   /* Use display_name as name instead since autoconfig display_name
+    * is destroyed already, and real name does not matter at this point */
+   if (input_st && !string_is_empty(input_st->input_device_info[port].display_name))
+      strlcpy(autoconfig_handle->device_info.name,
+            input_st->input_device_info[port].display_name,
+            sizeof(autoconfig_handle->device_info.name));
+   else if (!string_is_empty(name))
       strlcpy(autoconfig_handle->device_info.name,
             name, sizeof(autoconfig_handle->device_info.name));
 
    /* Configure task */
-   task = task_init();
-
-   if (!task)
+   if (!(task = task_init()))
       goto error;
 
    task->handler  = input_autoconfigure_disconnect_handler;
@@ -890,6 +903,21 @@ bool input_autoconfigure_disconnect(unsigned port, const char *name)
 
    task_queue_push(task);
 
+   if (pause_on_disconnect && core_is_running)
+   {
+#ifdef HAVE_MENU
+      bool menu_pause_libretro = settings->bools.menu_pause_libretro;
+      bool menu_is_alive       = menu_state_get_ptr()->flags & MENU_ST_FLAG_ALIVE;
+
+      if (menu_pause_libretro && !menu_is_alive)
+         command_event(CMD_EVENT_MENU_TOGGLE, NULL);
+      else if (!menu_pause_libretro)
+         command_event(CMD_EVENT_PAUSE, NULL);
+#else
+      command_event(CMD_EVENT_PAUSE, NULL);
+#endif
+   }
+
    return true;
 
 error:
@@ -901,7 +929,6 @@ error:
    }
 
    free_autoconfig_handle(autoconfig_handle);
-   autoconfig_handle = NULL;
 
    return false;
 }
